/*
 * (c) Copyright 2010-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.util.Locale;
import java.util.Set;

import org.databene.commons.BeanUtil;
import org.databene.commons.CollectionUtil;
import org.databene.commons.ConfigurationError;
import org.databene.commons.ConnectFailedException;
import org.databene.commons.FileUtil;
import org.databene.commons.IOUtil;
import org.databene.commons.ImportFailedException;
import org.databene.commons.StringUtil;
import org.databene.commons.SystemInfo;
import org.databene.commons.Visitor;
import org.databene.commons.ZipUtil;
import org.databene.commons.version.VersionNumber;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.CheckContext;
import org.databene.dbsanity.model.Reportable;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.model.TestSuiteLogger;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.parser.HierarchyProviderParser;
import org.databene.dbsanity.parser.SanityCheckSuiteParser;
import org.databene.dbsanity.parser.VersionProviderParser;
import org.databene.dbsanity.report.CheckAssessmentModule;
import org.databene.dbsanity.report.DefectsPerCheckModule;
import org.databene.dbsanity.report.DefectsPerTagModule;
import org.databene.dbsanity.report.DefectsPerTypeModule;
import org.databene.dbsanity.report.DefectsPerTableModule;
import org.databene.dbsanity.report.EmptyModule;
import org.databene.dbsanity.report.ErrorModule;
import org.databene.dbsanity.report.DefectsPageModule;
import org.databene.dbsanity.report.HTMLCheckExporter;
import org.databene.dbsanity.report.DigestModule;
import org.databene.dbsanity.report.InfoModule;
import org.databene.dbsanity.report.ReportContext;
import org.databene.dbsanity.report.ExecutionTimeModule;
import org.databene.dbsanity.report.SummaryReportsModule;
import org.databene.dbsanity.report.VerdictSummaryModule;
import org.databene.dbsanity.report.SubSuiteModule;
import org.databene.dbsanity.util.HierarchyBuilder;
import org.databene.html.HTMLUtil;
import org.databene.jdbacl.DBUtil;
import org.databene.jdbacl.JDBCConnectData;
import org.databene.jdbacl.VersionProvider;
import org.databene.jdbacl.model.Database;
import org.databene.jdbacl.model.cache.CachingDBImporter;
import org.databene.jdbacl.model.jdbc.JDBCMetaDataUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Main class that reads the arguments, performs the checks and creates a
 * report. As command line argument, an environment name is expected (e.g.
 * 'mydb'), which is mapped to a properties file name (e.g.
 * 'mydb.env.properties') and the JDBC connection info is read from it. By
 * default, the test definitions are expected in the working directory's sub
 * folder 'test' and the report is written to the working directory's sub folder
 * 'result'.<br/>
 * <br/>
 * Created: 13.10.2010 13:40:56
 * 
 * @since 1.0
 * @author Volker Bergmann
 */
public class DbSanity {

	private static final String TABLE_IN_CLOUD = "tableInCloud";

	private static final String AUTHOR_IN_CLOUD = "authorInCloud";

	private static final String EXECUTION_MODE = "executionMode";

	private static final Logger LOGGER = LoggerFactory.getLogger(DbSanity.class);
	
	static final String DEFAULT_CHECK_DEFINITION_FOLDER = "dbsanity";

	public static final String DEFAULT_REPORT_FOLDER = "dbsanity-report";

	static final String DEFAULT_TEST_TEMP_RESULT_FOLDER = "temp";

	public static final String DEFAULT_STYLE = "online";

	public static final int MAX_PATH_LENGTH = 150;

	private static final File USER_HOME_DATABENE = new File(SystemInfo.getUserHome(), "databene"); // TODO v0.9.5 move to commons
	private static final File USER_CONFIG_FILE = new File(USER_HOME_DATABENE, "dbsanity.xml");

	// attributes --------------------------------------------------------------

	String commandLine;
	File checkDefinitionFile;
	File tempFolder;
	File reportFolder;
	private String style = DEFAULT_STYLE;
	boolean clearBefore;
	boolean metaDataCached;
	boolean deleteTempAfter;
	boolean condensed;
	private ExecutionMode mode;
	private Locale locale;
	private String ignorableTables;
	private String[] tables;
	
	JDBCConnectData connectData;

	private String environment;

	private VersionProvider versionProvider;

	private boolean browsing;

	private int defectCountLimit;
	private int defectRowLimit;

	private Set<String> tags;
	private boolean zipping = false;
	private HierarchyProvider hierarchyProvider;

	private Connection connection;

	private int recursionDepth = 1;

	private boolean authorInCloud = true;

	private boolean tableInCloud = false;

	private boolean fallback = true;

	// constructors ------------------------------------------------------------

	public DbSanity() {
		this(new File(DEFAULT_CHECK_DEFINITION_FOLDER), null);
	}

	public DbSanity(File checkDefinitionFile, JDBCConnectData connectData) {
		this.checkDefinitionFile = checkDefinitionFile;
		this.tempFolder = new File(DEFAULT_TEST_TEMP_RESULT_FOLDER); // TODO v1.0 take the system's temp directory
		this.reportFolder = new File(DEFAULT_REPORT_FOLDER);
		this.connectData = connectData;
		this.browsing = false;
		this.clearBefore = false;
		this.metaDataCached = false;
		this.deleteTempAfter = true;
		this.condensed = false;
		this.mode = ExecutionMode.DEFAULT;
		this.locale = Locale.getDefault();
		this.defectCountLimit = 0;
		this.defectRowLimit = 0;
		this.ignorableTables = null;
		this.tables = null;
		this.hierarchyProvider = null;
	}

	// properties --------------------------------------------------------------

	public void setEnvironment(String environment) throws IOException {
		this.environment = environment;
		setConnectData(DBUtil.getConnectData(environment));
	}

	public JDBCConnectData getConnectData() {
		return connectData;
	}

	public void setConnectData(JDBCConnectData connectData) {
		this.connectData = connectData;
	}

	private Connection getConnection() throws ConnectFailedException {
		if (connection == null)
			connection = DBUtil.connect(connectData, true);
		return connection;
	}

	public void setConnection(Connection connection) {
		this.connection = connection;
	}

	public void setVersionProvider(VersionProvider versionProvider) {
		this.versionProvider = versionProvider;
	}
	
	public void setIgnorableTables(String ignorableTables) {
		this.ignorableTables = ignorableTables;
	}

	public File getCheckDefinitionFile() {
		return checkDefinitionFile;
	}

	public void setCheckDefinitionFile(File testDefinitionFolder) {
		this.checkDefinitionFile = testDefinitionFolder;
	}

	public void setTempFolder(File tempFolder) {
    	this.tempFolder = tempFolder;
    }

	public File getReportFolder() {
    	return reportFolder;
    }

	public void setReportFolder(File reportFolder) {
    	this.reportFolder = reportFolder;
    }

	public void setTables(String[] tables) {
		this.tables = tables;
	}

	public void setBrowsing(boolean browsing) {
		this.browsing = browsing;
	}

	public void setClearBefore(boolean clearBefore) {
    	this.clearBefore = clearBefore;
    }

	public void setMetaDataCached(boolean metaDataCached) {
		this.metaDataCached = metaDataCached;
	}
	
	public void setDeleteTempAfter(boolean deleteTempAfter) {
		this.deleteTempAfter = deleteTempAfter;
	}

	public void setCondensed(boolean condensed) {
		this.condensed = condensed;
	}
	
	public void setSkin(String style) {
	    this.style = style;
    }

	public ExecutionMode getMode() {
		return mode;
	}
	
	public void setMode(ExecutionMode mode) {
		this.mode = mode;
	}

	public Locale getLocale() {
		return locale;
	}
	
	public void setLocale(Locale locale) {
		this.locale = locale;
	}
	
	public void setHierarchyProvider(HierarchyProvider hierarchyProvider) {
		this.hierarchyProvider = hierarchyProvider;
	}
	
	public String getCommandLine() {
		return commandLine;
	}
	
	public void setCommandLine(String commandLine) {
		this.commandLine = commandLine;
	}
	
	
	
	// interface ---------------------------------------------------------------

	public SanityCheckSuite execute() throws Exception {
		checkPreconditions();

		// clear temp and report folders if they exist and shall be cleared
		if (clearBefore) {
			FileUtil.deleteDirectoryIfExists(tempFolder);
			FileUtil.deleteDirectoryIfExists(reportFolder);
		}
		
		// create temp folder beside test definition folder
		File tempFolder = new File(checkDefinitionFile.getParent(), DEFAULT_TEST_TEMP_RESULT_FOLDER); // TODO v0.x use system temp folder
		FileUtil.ensureDirectoryExists(tempFolder);

		parseConfigFileOrFolder(checkDefinitionFile, true);

		SanityCheckSuiteParser parser = new SanityCheckSuiteParser();
		SanityCheckSuite rootSuite = parser.parseHierarchy(checkDefinitionFile, reportFolder, tempFolder);

	    ReportContext context = createContext(rootSuite.getName());

	    // create HTML versions of the check definitions
		renderSuiteInHTML(rootSuite, context);

		// apply hierarchy provider
		SanityCheckSuite hierarchyRoot = applyHierarchy(rootSuite);
		
		execute(hierarchyRoot, context);
		new TestSuiteLogger().log(hierarchyRoot);

		LOGGER.info("Sanity check finished");
		String reportURL = reportFolder.getCanonicalPath() + File.separator + "index.html";
		
		if (browsing)
			HTMLUtil.openBrowser("file://" + reportURL);
		
		if (zipping)
			reportURL = zipReport(context);
		if (mode != ExecutionMode.QUIET)
			System.out.println("The DBSanity report is available at " + reportURL);
		return hierarchyRoot;
	}

	private SanityCheckSuite applyHierarchy(SanityCheckSuite rootSuite) {
		return HierarchyBuilder.buildHierarchy(hierarchyProvider, rootSuite);
	}

	public ReportContext execute(SanityCheckSuite rootSuite, ReportContext context) throws Exception {
		// read database meta data and application version
		Connection connection = getConnection();		
		Database database = getDatabaseMetaData(connection);
		VersionNumber appVersion = (versionProvider != null ? versionProvider.getVersion(connection) : null);

		// parse checks
		rootSuite.checkApplicability(appVersion, CollectionUtil.<String>toSet(tables), tags);
		
	    context.setAppVersion(appVersion);

	    Reporter reporter = createReporter(context, database);

	    try {
		    // perform checks
			CheckContext checkContext = new CheckContext(connection, database, context.getResourceBundle(), rootSuite.countPendingChecks());
			rootSuite.reset();
			rootSuite.perform(checkContext, reporter);
			reportFolder.mkdir();
			// create summary
			reporter.summary(rootSuite);
		} finally {
			// close everything
			DBUtil.close(connection);
		    IOUtil.close(reporter);
		    if (deleteTempAfter)
		    	FileUtil.deleteDirectoryIfExists(tempFolder);
		    CachingDBImporter.updateCacheFile(database);
		}
		return context;
	}

	protected Database getDatabaseMetaData(Connection connection)
			throws ConnectFailedException, ImportFailedException {
		Database database;
		if (environment != null)
			database = JDBCMetaDataUtil.getMetaData(environment, false, false, false, false, null, ignorableTables, true, metaDataCached);
		else
			database = JDBCMetaDataUtil.getMetaData(connection, connectData.user, connectData.schema, false, false, false, false, null, ignorableTables);
		return database;
	}

	private String zipReport(ReportContext context) throws IOException {
		File reportRoot = context.getReportRoot();
		File zipFile = new File(reportRoot.getParentFile(), reportRoot.getName() + ".zip");
		String path = zipFile.getCanonicalPath();
		LOGGER.info("Compressing report into file " + path);
		ZipUtil.compress(reportRoot, zipFile);
		if (!browsing)
			FileUtil.deleteDirectory(reportRoot);
		return path;
	}

	private void renderSuiteInHTML(SanityCheckSuite suite, final ReportContext context) throws IOException {
		final HTMLCheckExporter exporter = new HTMLCheckExporter();
		Visitor<Reportable> visitor = new Visitor<Reportable>() {
			public void visit(Reportable element) {
				if (element instanceof SanityCheckFile) {
					SanityCheckFile file = (SanityCheckFile) element;
					if (element instanceof SanityCheckFile)
						try {
							exporter.export(file, context, style);
						} catch (IOException e) {
							throw new RuntimeException("Error exporting file in HTML: " + file.getSource(), e);
						}
				}
			}
		};
		suite.accept(visitor);
	}

	private boolean parseConfigFileOrFolder(File fileOrFolder, boolean isSpec) throws IOException {
		boolean found = false;
		if (fileOrFolder.exists()) {
			if (fileOrFolder.isDirectory())
				found = parseConfigFileOrFolder(new File(fileOrFolder, "dbsanity.xml"), false);
			else if (fileOrFolder.isFile() && (isSpec || fileOrFolder.getName().equals("dbsanity.xml"))) {
				parseConfigFile(fileOrFolder);
				found = true;
			}
		}
		if (!found && fallback && !USER_CONFIG_FILE.equals(fileOrFolder))
			return parseConfigFileOrFolder(USER_CONFIG_FILE, false);
		return found;
	}

	public void parseConfigFile(File file) throws IOException {
		// TODO v1.0 don't replicate parsing logic here
		Document doc = XMLUtil.parse(file.getAbsolutePath());
		Element root = doc.getDocumentElement();
		// ignorableTables
		if (this.ignorableTables == null) {
			// if inorableTables has been specified explicitly, it overrides the settings in the config file
			String ignorableTables = root.getAttribute("ignorableTables");
			if (!StringUtil.isEmpty(ignorableTables))
				this.ignorableTables = ignorableTables;
		}
		// recursionDepth
		String recursionDepth = root.getAttribute("recursionDepth");
		if (!StringUtil.isEmpty(recursionDepth))
			this.recursionDepth = Integer.parseInt(recursionDepth);
		// authorInCloud
		String authorInCloud = root.getAttribute(AUTHOR_IN_CLOUD);
		if (authorInCloud != null)
			this.authorInCloud = Boolean.parseBoolean(authorInCloud);
		// tableInCloud
		String tableInCloud = root.getAttribute(TABLE_IN_CLOUD);
		if (tableInCloud != null)
			this.tableInCloud = Boolean.parseBoolean(tableInCloud);

		// versionProvider
		Element versionProviderElement = XMLUtil.getChildElement(root, false, false, "versionProvider");
		if (versionProviderElement != null)
			this.versionProvider = new VersionProviderParser().parse(versionProviderElement, null, null);
		// hierarchyProvider
		Element hierarchyProviderElement = XMLUtil.getChildElement(root, false, false, "hierarchyProvider");
		if (hierarchyProviderElement != null)
			this.hierarchyProvider = new HierarchyProviderParser().parse(hierarchyProviderElement, null, null);
	}

	// private helpers ---------------------------------------------------------

	private ReportContext createContext(String rootSuiteName) {
		ReportContext context = new ReportContext(rootSuiteName, reportFolder, tables, locale);
		context.set("systemUser", SystemInfo.getUserName());
		context.set("connectData", connectData);
		context.setIgnorableTables(ignorableTables);
		context.setTablesToCheck(tables);
		context.setTags(tags);
		context.setDefectCountLimit(defectCountLimit);
		context.setDefectRowLimit(defectRowLimit);
	    context.set(EXECUTION_MODE, mode);
	    context.setAuthorInCloud(authorInCloud);
	    context.setTableInCloud(tableInCloud);
	    context.setCondensed(condensed);
	    context.set("commandLine", commandLine);
	    return context;
    }

	private Reporter createReporter(ReportContext context, Database database) throws IOException {
	    Reporter reporter = new Reporter(reportFolder, style, context);
	    String className = "org.databene.dbsanity.report." + StringUtil.capitalize(style) + "VerdictSummaryModule";
	    VerdictSummaryModule summaryModule = (VerdictSummaryModule) BeanUtil.newInstance(className);
	    
	    reporter.addModule(summaryModule);
		reporter.addModule(new InfoModule());

	    reporter.addModule(new DefectsPerTagModule());
	    reporter.addModule(new CheckAssessmentModule(database, ignorableTables));

	    reporter.addModule(new DigestModule());
	    reporter.addModule(new SummaryReportsModule());

	    reporter.addModule(new DefectsPerTableModule());
	    reporter.addModule(new EmptyModule());

	    reporter.addModule(new DefectsPerTypeModule());
	    
	    reporter.addModule(new DefectsPerCheckModule());

	    reporter.addModule(new SubSuiteModule(recursionDepth, condensed));
	    
	    reporter.addModule(new ExecutionTimeModule());

	    reporter.addModule(new DefectsPageModule());
	    reporter.addModule(new ErrorModule());
	    //reporter.addModule(new TempResultFileModule());
	    reporter.setContext(context);
	    return reporter;
    }

	private void checkPreconditions() {
	    if (checkDefinitionFile == null)
			throw new ConfigurationError(getClass().getSimpleName()
					+ ".testDefinitionFolder is null");
		if (!checkDefinitionFile.exists())
			throw new ConfigurationError("Test definition folder not found: "
					+ checkDefinitionFile.getAbsolutePath());
		if (connectData == null && connection == null)
			throw new ConfigurationError("No database specified");
    }

	public void setDefectCountLimit(int defectCountLimit) {
		this.defectCountLimit = defectCountLimit;
	}

	public void setDefectRowLimit(int defectRowLimit) {
		this.defectRowLimit = defectRowLimit;
	}
	
	public void setTags(Set<String> tags) {
		this.tags = tags;
	}

	public void setZipping(boolean zipping) {
		this.zipping = zipping;
	}

	public void setFallback(boolean fallback) {
		this.fallback = fallback;
	}

}
