/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.html;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.util.Stack;

import org.databene.commons.Encodings;
import org.databene.commons.FileUtil;
import org.databene.commons.file.FilePrintStream;
import org.databene.commons.file.FilePrintWriter;
import org.databene.html.HTMLUtil;

/**
 * {@link FilePrintStream} implementation which adds methods for HTML rendering.<br/><br/>
 * Created: 01.12.2011 21:50:13
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class HtmlPrintWriter extends FilePrintWriter {
	
	private boolean unclosed;
	private String indent;
	private Stack<String> path;

	public HtmlPrintWriter(File file, String title, HtmlStyle... styles) 
			throws FileNotFoundException, UnsupportedEncodingException {
		this(file, Encodings.UTF_8, title, styles);
	}
	
	public HtmlPrintWriter(File file, String encoding, String title, HtmlStyle... styles) 
			throws FileNotFoundException, UnsupportedEncodingException {
		super(file, encoding);
		this.unclosed = true;
		this.indent = "";
		this.path = new Stack<String>();
	    startContent(title, styles);
	}
	
	// interface -------------------------------------------------------------------------------------------------------
	
	@Override
	public void close() {
		if (unclosed) {
			this.unclosed = false;
			endContent();
		    super.close();
		}
	}

	public void startModule(String title) {
		startPanel("class", "module");
		startPanel("class", "mod_header");
		printTextInTag("h3", title);
		endPanel(); // mod_header
		startPanel("class", "mod_body");
	}
	
	public void printModuleSeparator(String title) {
		endPanel(); // mod_body
		startPanel("class", "mod_header");
		printTextInTag("h3", title);
		endPanel(); // mod_header
		startPanel("class", "mod_body");
	}
	
	public void endModule() {
		endModule(null, null);
	}

	public void endModule(String label, File detailsFile) {
		endPanel(); // mod_body
		if (detailsFile != null) {
			startPanel("class", "mod_footer");
			render(new HtmlFileReference(label, detailsFile));
			endPanel();
		}
		endPanel(); // module
	}
	
	public void printInlinePanel(Renderable content, String... nameValuePairs) {
		printInTag("div", content, nameValuePairs);
	}
	
	public void startPanel(String... nameValuePairs) {
		startTag("div", nameValuePairs);
	}
	
	public void endPanel() {
		endTag("div");
	}
	
	public void startTable(String... nameValuePairs) {
		startTag("table", nameValuePairs);
	}
	
	public void endTable() {
		endTag("table");
	}
	
	public void startTableHead() {
		startTag("thead");
	}
	
	public void endTableHead() {
		endTag("thead");
	}
	
	public void startTableBody() {
		startTag("tbody");
	}
	
	public void endTableBody() {
		endTag("tbody");
	}
	
	public void renderRow(HtmlTableCell... cells) {
		startTableRow();
		for (HtmlTableCell cell : cells)
			cell.render(this);
		endTableRow();
	}
	
	public void startTableRow(String... nameValuePairs) {
		startTag("tr", nameValuePairs);
	}
	
	public void endTableRow() {
		endTag("tr");
	}
	
	public void printTableHeaderCell(String text, String... nameValuePairs) {
		startTag("th", true, nameValuePairs);
		print(text);
		endTag("th");
	}
	
	public void startTableHeaderCell(String... nameValuePairs) {
		startTag("th", nameValuePairs);
	}
	
	public void endTableHeaderCell() {
		endTag("th");
	}
	
	public void printTableCell(String text, String... nameValuePairs) {
		startTag("td", true, nameValuePairs);
		print(text);
		endTag("td");
	}
	
	public void startTableCell(String... nameValuePairs) {
		startTag("td", nameValuePairs);
	}
	
	public void endTableCell() {
		endTag("td");
	}
	
	public void printLineBreak() {
		println("<br/>");
	}
	
	public void startPreformatted() {
		startTag("pre");
	}

	public void endPreformatted() {
		endTag("pre");
	}

	public void render(Renderable... renderables) {
		for (Renderable renderable : renderables)
			renderable.render(this);
	}

	public void startParagraph(String... nameValuePairs) {
		startTag("p", nameValuePairs);
	}

	public void endParagraph() {
		endTag("p");
	}

	public void startBold() {
		startTag("b");
	}

	public void endBold() {
		endTag("b");
	}

	public void printText(String text) {
		println(HTMLUtil.escape(text));
	}
	
	// private helpers -------------------------------------------------------------------------------------------------
	
	protected void startContent(String title, HtmlStyle... styles) {
		println(HTMLUtil.DOCTYPE_401);
	    startTag("html");
	    startTag("head");
	    println("		<title>" + title + "</title>");
	    println("		<meta http-equiv='Content-Type' content='text/html; charset=" + encoding + "' />");
	    for (HtmlStyle style : styles) {
	    	String cssRef = FileUtil.relativePath(file, style.getFile(), '/');
		    println("		<link href='" + cssRef + "' media='all' rel='stylesheet' type='text/css' />");
	    }
	    endTag("head");
	    startTag("body");
	    startPanel("class", "page_header");
	    printTextInTag("h1", title);
	    endPanel();
	    startTag("center");
	}
	
	protected void endContent() {
		endTag("center");
		endTag("body");
		endTag("html");
	}
	
	protected void printTextInTag(String tag, String text, String... nameValuePairs) {
		printInTag(tag, new RenderableText(text), nameValuePairs);
	}
	
	protected void printInTag(String tag, Renderable content, String... nameValuePairs) {
		print(indent);
		print("<");
		print(tag);
		for (int i = 0; i < nameValuePairs.length; i += 2) {
			print(' ');
			print(nameValuePairs[i]);
			print("=\"");
			print(nameValuePairs[i + 1]);
			print('"');
		}
		print('>');
		content.render(this);
		print("</");
		print(tag);
		print('>');
		println();
	}
	
	public void startTag(String name, String... nameValuePairs) {
		startTag(name, false, nameValuePairs);
	}

	public void startTag(String name, boolean inline, String... nameValuePairs) {
		this.path.add(name);
		if (!inline)
			print(indent);
		print("<");
		print(name);
		for (int i = 0; i < nameValuePairs.length; i += 2) {
			print(' ');
			print(nameValuePairs[i]);
			print("=\"");
			print(nameValuePairs[i + 1]);
			print('"');
		}
		print('>');
		if (!inline) {
			println();
			this.indent += '\t';
		}
	}

	public void endTag(String tag) {
		endTag(tag, false);
	}

	public void endTag(String tag, boolean inline) {
		if (!inline) {
			if (this.indent.length() > 0)
				this.indent = indent.substring(0, indent.length() - 1);
			print(indent);
		}
		print("</");
		String pathTag = path.pop();
		if (!tag.equals(pathTag))
			throw new IllegalStateException("Closing tag failed for " + super.toString() + ": Expected </" + tag + ">, but was </" + pathTag + ">");
		print(pathTag);
		print('>');
		if (!inline)
			println();
	}

}
