/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.html;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import org.databene.commons.ConfigurationError;

/**
 * {@link MessageFormat}-like utility class which supports {@link Renderable}s as variables.<br/><br/>
 * Created: 08.12.2011 11:38:47
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class RenderableMessageFormat {

	public static void format(String pattern, HtmlPrintWriter out, Renderable... renderables) {
		assureEachPlaceholderIsUsed(pattern, renderables);
		List<Object> tokens = splitTokens(pattern);
		for (Object token : tokens) {
			if (token instanceof String)
				out.printText((String) token);
			else
				out.render(renderables[(Integer) token]);
		}
	}

	static List<Object> splitTokens(String pattern) {
		List<Object> tokens = new ArrayList<Object>();
		if (pattern == null)
			return tokens;
		String rest = pattern;
		while (rest.length() > 0) {
			int lBracketIndex = rest.indexOf('{');
			if (lBracketIndex > 0) {
				// extract leading text and reduce to remaining pattern
				tokens.add(rest.substring(0, lBracketIndex));
				rest = rest.substring(lBracketIndex);
				lBracketIndex = 0;
			}
			if (lBracketIndex == 0) {
				// There is a leading '{'
				int rBracketIndex = rest.indexOf('}', lBracketIndex);
				if (rBracketIndex >= 0) {
					try {
						String indexString = rest.substring(lBracketIndex + 1, rBracketIndex);
						int variableIndex = Integer.parseInt(indexString);
						tokens.add(variableIndex);
					} catch (IllegalArgumentException e) {
						tokens.add(rest.substring(lBracketIndex, rBracketIndex + 1));
					}
					rest = rest.substring(rBracketIndex + 1);
				} else {
					tokens.add("{");
					rest = rest.substring(1);
				}
			} else {
				tokens.add(rest);
				rest = "";
			}
				
		}
		if (rest.length() > 0)
			tokens.add(rest);
		return tokens;
	}

	private static void assureEachPlaceholderIsUsed(String pattern, Renderable... renderables) {
		for (int i = 0; i < renderables.length; i++)
			if (pattern.indexOf("{" + i + "}") < 0)
				throw new ConfigurationError("Missing placeholder {" + i + "} in pattern: " + pattern);
	}

}
