/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import org.databene.commons.ArrayUtil;
import org.databene.commons.ObjectNotFoundException;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.SystemInfo;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.jdbacl.DatabaseDialect;
import org.databene.jdbacl.DatabaseDialectManager;
import org.databene.jdbacl.SQLUtil;
import org.databene.jdbacl.model.DBForeignKeyConstraint;
import org.databene.jdbacl.model.DBTable;
import org.databene.jdbacl.model.Database;

/**
 * Verifies that a many-to-one association has cardinalities in a given range.<br/><br/>
 * Created: 05.11.2011 07:07:45
 * @since 0.9.1
 * @author Volker Bergmann
 */
public class CardinalityQueryStrategy extends DefectQueryStrategy {

	private String refererTableName;
	private String[] refererColumns;
	private String refereeTableSpec;
	private String refereeColumnsSpec;
	
	private String reportColumnsSpec;
	private Long min;
	private Long max;
	
	private String code;

	public CardinalityQueryStrategy(String refererTableName,
			String[] refererColumns, String refereeTableSpec,
			String refereeColumnsSpec, String reportColumnsSpec,
			Long min, Long max,
			String code) {
		super(null, "cardinality");
		this.refererTableName = refererTableName;
		this.refererColumns = refererColumns;
		this.refereeTableSpec = refereeTableSpec;
		this.refereeColumnsSpec = refereeColumnsSpec;
		this.reportColumnsSpec = reportColumnsSpec;
		this.min = min;
		this.max = max;
		this.code = code;
	}

	@Override
	public DefectIterator perform(CheckContext context) {
		Database database = context.getDatabase();
		DBTable refererTable = database.getTable(refererTableName);
		DBTable refereeTable;
		String refereeTableName = null;
		String[] refereeColumns = null;
		if (refereeTableSpec != null && refereeColumnsSpec != null) {
			refereeTableName = refereeTableSpec;
			refereeTable = database.getTable(refereeTableName);
			refereeColumns = refereeColumnsSpec.split(",");
			StringUtil.trimAll(refereeColumns);
		} else {
			try {
				// if refereeTable and/or refereeColumns are missing, try to look them up in database metadata
				DBForeignKeyConstraint fkConstraint = refererTable.getForeignKeyConstraint(refererColumns);
				refereeTable = fkConstraint.getRefereeTable();
				String autoRefereeTableName = refereeTable.getName();
				String[] autoRefereeColumns = fkConstraint.getRefereeColumnNames();
				if (refereeTableSpec != null && !refereeTableSpec.equalsIgnoreCase(autoRefereeTableName))
					throw new SyntaxError("'refereeTable' specification inconsistent with foreign key in database", code);
				refereeTableName = autoRefereeTableName;
				if (refereeColumnsSpec != null && !StringUtil.equalsIgnoreCase(autoRefereeColumns, StringUtil.trimAll(refereeColumnsSpec.split(","))))
					throw new SyntaxError("'refereeColumns' specification inconsistent with foreign key in database", code);
				refereeColumns = autoRefereeColumns;
			} catch (ObjectNotFoundException e) {
				// if no fk metadata exists and only refereeColumns are missing, assume primary key columns
				if (refereeTableSpec == null)
					throw new SyntaxError("'refereeTable' is missing", code);
				refereeTable = database.getTable(refereeTableName);
				refereeColumns = refereeTable.getPKColumnNames();
				if (!ArrayUtil.isEmpty(refereeColumns))
					throw new SyntaxError("'refereeColumns' is missing", code);
			}
		}
		
		if (min != null & max != null && min > max)
			throw new SyntaxError("min greater than max", code);
		
		// create query
		DatabaseDialect dialect = DatabaseDialectManager.getDialectForProduct(
				database.getDatabaseProductName(), database.getDatabaseProductVersion());
	    String LF = SystemInfo.getLineSeparator();
		String reportColumnList = renderReportColumns(reportColumnsSpec, refereeTable);
		String[] refererPkColumns = refererTable.getPKColumnNames();
		if (refererPkColumns.length == 0)
			throw new IllegalArgumentException("Table " + refereeTableName + " must have a primary key to perform this check");
		String defectQuery = "select " + reportColumnList + ", " + 
			dialect.renderCase("_ref_count", "_left_count", "_min_referer_id is null", "0") +
			" from (" + LF +
				"\tselect " +  LF +
					"\t\t" + SQLUtil.renderColumnListWithTableName("referee", reportColumnList) + ", " + LF +
					"\t\tmin(referer." + refererPkColumns[0] + ") as _min_referer_id, " + LF +
					"\t\tcount(*) as _left_count " + LF +
				"\tfrom " + refereeTableName + " as referee " + LF +
				"\t\t" + SQLUtil.leftJoin("referee", refereeColumns, refererTable.getName(), "referer", refererColumns) + " " + LF +
				"\tgroup by " + SQLUtil.renderColumnListWithTableName("referee", reportColumnList) + " " + LF +
			") where _ref_count ";
		if (min != null && max != null)
			defectQuery += "not between " + min + " and " + max;
		else if (min != null)
			defectQuery += " < " + min;
		else
			defectQuery += " > " + max;
		this.query = defectQuery;
		return super.perform(context);
	}

}
