/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.util.HashMap;
import java.util.Map;

import org.databene.commons.ArrayFormat;
import org.databene.commons.StringUtil;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.jdbacl.SQLUtil;
import org.databene.jdbacl.model.DBTable;
import org.xml.sax.SAXException;

/**
 * Verifies that a columns values can be represented with a given number of digits 
 * or are within a range from minimum to maximum.<br/><br/>
 * Created: 05.11.2011 07:43:37
 * @since 0.9.1
 * @author Volker Bergmann
 */
public class NumberSizeQueryStrategy extends DefectQueryStrategy {

	private String tableName;
	private String expression;
	private String condition;
	private Integer totalDigits;
	private Integer fractionDigits;
	private String min;
	private String max;
	private String reportColumnsSpec;

	public NumberSizeQueryStrategy(String tableName, String expression, String condition,
			Integer totalDigits, Integer fractionDigits, String min,
			String max, String reportColumnsSpec) {
		super(null, "numberSize");
		this.tableName = tableName;
		this.expression = expression;
		this.condition = condition;
		this.totalDigits = totalDigits; 
		this.fractionDigits = fractionDigits; 
		this.min = min;
		this.max = max; 
		this.reportColumnsSpec = reportColumnsSpec;
	}

	@Override
	public DefectIterator perform(CheckContext context) {
		// create query
		DBTable table = context.getDatabase().getTable(tableName);
		StringBuilder sizeCondition = new StringBuilder();
		if (min != null)
			SQLUtil.addOptionalCondition(expression + " < " + min, sizeCondition);
		if (max != null)
			SQLUtil.addOptionalCondition(expression + " > " + max, sizeCondition);
		int postfixDigits = (fractionDigits != null ? fractionDigits : 0);
		if (totalDigits != null)
			SQLUtil.addOptionalCondition(expression + " > POWER(10, " + (totalDigits - postfixDigits) + ")", sizeCondition);
		if (fractionDigits != null)
			SQLUtil.addOptionalCondition(expression + " != round(" + expression + ", " + postfixDigits + ")", sizeCondition);
		String[] reportColumns = parseReportColumns(reportColumnsSpec, table);
		reportColumns = appendColumnIfNotIncluded(expression, reportColumns);
		this.query = "SELECT " + ArrayFormat.format(reportColumns) + " FROM " + table + " WHERE " + sizeCondition.toString();
		if (!StringUtil.isEmpty(condition))
			this.query += " AND " + condition;
		return super.perform(context);
	}
	
	@Override
	public void serialize(SimpleXMLWriter writer) throws SAXException {
		Map<String, String> atts = new HashMap<String, String>();
		atts.put("expression", expression);
		atts.put("condition", condition);
		if (totalDigits != null)
			atts.put("totalDigits", totalDigits.toString());
		if (fractionDigits != null)
			atts.put("fractionDigits", totalDigits.toString());
		atts.put("reportColumnsSpec", reportColumnsSpec);
		if (min != null)
			atts.put("min", String.valueOf(min));
		if (max != null)
			atts.put("max", String.valueOf(max));
		writer.startElement("numberSize", atts);
		writer.endElement("numberSize");
	}
	
	@Override
	public String toString() {
		return "numberSize[" + tableName + '.' + expression + ", " + 
			(totalDigits != null ? "totalDigits:" + totalDigits.toString() + ", " : "") + 
			(fractionDigits != null ? "fractionDigits:" + fractionDigits.toString() + ", " : "") + 
			(min != null ? "min:" + min.toString() + ", " : "") + 
			(max != null ? "max:" + max.toString() + ", " : "") + 
			"]";
	}
	
}
