/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.util.HashMap;
import java.util.Map;

import org.databene.commons.ArrayFormat;
import org.databene.commons.ObjectNotFoundException;
import org.databene.commons.StringUtil;
import org.databene.commons.SystemInfo;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.jdbacl.SQLUtil;
import org.databene.jdbacl.model.DBForeignKeyConstraint;
import org.databene.jdbacl.model.DBTable;
import org.databene.jdbacl.model.Database;
import org.databene.jdbacl.model.ForeignKeyPath;
import org.xml.sax.SAXException;

/**
 * {@link DefectQueryStrategy} for checking the equality of foreign key path ends.<br/><br/>
 * Created: 23.03.2012 07:31:53
 * @since 0.9.4
 * @author Volker Bergmann
 */
public class PathsQueryStrategy extends DefectQueryStrategy {

	private String referenceSpec;
	private String testeeSpec;
	private boolean equal;
	private ForeignKeyPath reference;
	private ForeignKeyPath testee;
	private String condition;
	private String reportColumnsSpec;

	public PathsQueryStrategy(ForeignKeyPath reference, ForeignKeyPath testee, boolean equal, 
			String condition, String reportColumnsSpec) {
		super(null, "routes");
		this.reference = reference;
		this.testee = testee;
		this.equal = equal;
		this.condition = condition;
		this.reportColumnsSpec = reportColumnsSpec;
	}

	public PathsQueryStrategy(String referenceSpec,
			String testeeSpec, boolean equal, String condition, String reportColumnsSpec) {
		super(null, "routes");
		this.referenceSpec = referenceSpec;
		this.testeeSpec = testeeSpec;
		this.equal = equal;
		this.condition = condition;
		this.reportColumnsSpec = reportColumnsSpec;
	}
	
	@Override
	public DefectIterator perform(CheckContext context) {
		this.query = createQuery(context);
		return super.perform(context);
	}

	@Override
	public void serialize(SimpleXMLWriter writer) throws SAXException {
		Map<String, String> atts = new HashMap<String, String>();
		atts.put("reference", reference.toString());
		atts.put("testee", testee.toString());
		atts.put("equal", String.valueOf(equal));
		atts.put("condition", condition);
		atts.put("reportColumnsSpec", reportColumnsSpec);
		writer.startElement("paths", atts);
		writer.endElement("paths");
	}
	
	@Override
	public String toString() {
		String LF = SystemInfo.getLineSeparator();
		return getClass().getSimpleName() + '[' + LF + 
			reference + LF +
			testee + LF;
	}
	
	
	
	// non-public helpers ----------------------------------------------------------------------------------------------
	
	private String createQuery(CheckContext context) {
		Database database = context.getDatabase();
		if (reference == null) {
			this.reference = parseFKPath(referenceSpec, database);
			this.testee = parseFKPath(testeeSpec, database);
		}
		// check which columns to write into the defect report
		String reportColumns;
		if (database != null) {
			DBTable startTable = database.getTable(testee.getStartTable());
			reportColumns = renderReportColumns(reportColumnsSpec, startTable);
		} else if (reportColumnsSpec != null)
			reportColumns = reportColumnsSpec;
		else
			reportColumns = "*";	
		// render the query
		String refJoin = SQLUtil.joinFKPath(reference, null, "_startpoint", "_endpoint1", "_ref");
		String testeeJoin = SQLUtil.joinFKPath(testee, null, "_startpoint", "_endpoint2", "_test");
		return "SELECT " + reportColumns + " FROM " + reference.getStartTable() + " _startpoint " + 
			refJoin + " " + 
			testeeJoin + " WHERE " + 
			condition + " AND " + renderEndPointComparison();
	}
	
	private String renderEndPointComparison() {
		String condition = SQLUtil.equals("_endpoint1", reference.getEndColumnNames(), 
				"_endpoint2", testee.getEndColumnNames());
		if (equal)
			condition = "NOT (" + condition + ")";
		return condition;
	}

	static ForeignKeyPath parseFKPath(String spec, Database database) {
		String[] nodes = spec.split(" \\->");
		ForeignKeyPath path = new ForeignKeyPath();
		for (int i = 0; i < nodes.length - 1; i++)
			path.addEdge(parseFK(nodes[i], database));
		return path;
	}

	private static DBForeignKeyConstraint parseFK(String spec, Database database) {
		spec = spec.trim();
		int iBracket = spec.indexOf('(');
		String tableName = spec.substring(0, iBracket);
		String columnList = spec.substring(iBracket + 1, spec.length() - 1);
		String[] columns = StringUtil.splitAndTrim(columnList, ',');
		DBTable refererTable = database.getTable(tableName, true);
		DBForeignKeyConstraint fk = refererTable.getForeignKeyConstraint(columns);
		if (fk == null)
			throw new ObjectNotFoundException("Foreign ke constraint not found: " + tableName + 
					'(' + ArrayFormat.format(columns) + ')');
		return fk;
	}

}
