/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.util.HashMap;
import java.util.Map;

import org.databene.commons.ArrayFormat;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.validator.RegexValidator;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.dbsanity.parser.Executor;
import org.databene.jdbacl.DatabaseDialect;
import org.databene.jdbacl.DatabaseDialectManager;
import org.databene.jdbacl.model.DBTable;
import org.databene.jdbacl.model.Database;
import org.xml.sax.SAXException;

/**
 * Verifies that an expression either fulfills a regular expression or a 'like' pattern.<br/><br/>
 * Created: 05.11.2011 09:34:17
 * @since 0.9.1
 * @author Volker Bergmann
 */
public class PatternStrategy extends CheckStrategyProxy {

	private String tableName;
	private String expression;
	private String condition;
	private String regex;
	private String like;
	private String reportColumnsSpec;
	private Executor executor;
	private String sourceCode;
	
	public PatternStrategy(String tableName, String expression, String condition, String regex, 
			String like, Executor executor, String reportColumnsSpec, String sourceCode) {
		super(null);
		this.tableName = tableName;
		this.expression = expression;
		this.condition = condition;
		this.regex = regex;
		this.like = like;
		this.executor = executor;
		this.reportColumnsSpec = reportColumnsSpec;
		this.sourceCode = sourceCode;
	}
	
	@Override
	public String getDefectType() {
		return "pattern";
	}
	
	public DefectIterator perform(CheckContext context) {
		initRealStrategy(context.getDatabase());
		return super.perform(context);
	}

	private void initRealStrategy(Database database) {
		// create query
		DatabaseDialect dialect = DatabaseDialectManager.getDialectForProduct(
				database.getDatabaseProductName(), database.getDatabaseProductVersion());
		DBTable table = database.getTable(tableName);
		String reportColumns[] = parseReportColumns(reportColumnsSpec, table);
		reportColumns = appendColumnIfNotIncluded(expression, reportColumns);
		String defectQuery = "SELECT " + ArrayFormat.format(reportColumns)  + " FROM " + table + " WHERE ";
		if (like != null)
			defectQuery += expression + " NOT LIKE '" + like + "'";
		if (!StringUtil.isEmpty(regex)) {
			// a regex has to be checked
			if (like != null)
				throw new SyntaxError("'regex' and 'like' constraints exclude each other", sourceCode);
			if (dialect.supportsRegex() && executor == Executor.DATABASE) {
				// if the database is able to perform regex checking and the user does not prevent it,
				// let the database do the regex check
				defectQuery += dialect.regexQuery(expression, true, regex);
			} else {
				// if the database is not capable of regex checking or the user explicitly configured 
				// client-side checking then use a client-side validator
				ValidatorStrategy strategy = new ValidatorStrategy(
						new RegexValidator(regex, true), table.getName(), new String[] { expression }, condition, reportColumns);
				this.realStrategy = strategy;
			}
		}
		if (this.realStrategy == null) {
			if (!StringUtil.isEmpty(condition))
				defectQuery += " AND " + condition;
			this.realStrategy = new DefectQueryStrategy(defectQuery, "pattern");
		}
	}
	
	@Override
	public void serialize(SimpleXMLWriter writer) throws SAXException {
		Map<String, String> atts = new HashMap<String, String>();
		atts.put("expression", expression);
		atts.put("regex", expression);
		atts.put("like", expression);
		if (executor != null)
			atts.put("executor", executor.name());
		atts.put("condition", condition);
		atts.put("reportColumns", reportColumnsSpec);
		writer.startElement("pattern", atts);
		writer.endElement("pattern");
	}
	
	@Override
	public String toString() {
		return "pattern[" + tableName + '.' + expression + " - " + regex + "]";
	}
	
}
