/*
 * (c) Copyright 2010-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.FileUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.Visitor;
import org.databene.commons.math.Intervals;
import org.databene.commons.version.VersionNumber;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.DbSanity;
import org.databene.dbsanity.report.ReportUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

/**
 * Represents a sanity check composed of a test name and a SQL query which is assumed to 
 * return an empty result set. Any data row accessible from the iterator returned by 
 * the perform() method is interpreted as one faulty data set.<br/><br/>
 * Created: 13.10.2010 13:07:08
 * @since 1.0
 * @author Volker Bergmann
 */
public abstract class SanityCheck implements Reportable {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(SanityCheck.class);
	
	// configuration attributes
	protected String name;
	protected String requirement;
	protected String description;
	protected String tableName;
	protected Intervals<VersionNumber> affectedVersions;
	protected String defectType;
	protected String author;

	private File sourceFile;
	private File tmpResultFile;
	private File reportFolder;
	private File docPage;
	private File defectsPage;
	
	// result attributes
	protected int executionTime;
	protected SanityCheckVerdict verdict;

	private String executionNote;

	private Set<String> tags;

	private StatusChangeSupport changeSupport;

	public SanityCheck(String name, String table, Intervals<VersionNumber> affectedVersions, String defectType, File sourceFile, File tempFolder, File reportFolder, File docPage) {
		// check preconditions
		if (StringUtil.isEmpty(name))
			throw new IllegalArgumentException("No name specified for " + getClass().getSimpleName());
		if (table == null)
			LOGGER.warn("No table specified for check '{}'", name);
		
		// initialize attributes
	    this.name = name;
	    this.tableName = table;
	    this.affectedVersions = affectedVersions;
	    this.docPage = docPage;
	    this.defectType = defectType;
	    if (reportFolder != null) {
	    	this.reportFolder = reportFolder;
		    this.defectsPage = FileUtil.fileOfLimitedPathLength(
		    		reportFolder, ReportUtil.normalizeUrl(name), ".defects.html", DbSanity.MAX_PATH_LENGTH, false);
	    }
	    this.sourceFile = sourceFile;
	    if (tempFolder != null) {
		    this.tmpResultFile = FileUtil.fileOfLimitedPathLength(
		    		tempFolder, ReportUtil.normalizeUrl(getName()), ".result", DbSanity.MAX_PATH_LENGTH, false);
			if (tmpResultFile.exists())
				tmpResultFile.delete();
	    }
		this.executionTime = 0;
	    this.verdict = SanityCheckVerdict.PENDING;
	    this.tags = new HashSet<String>();
	    this.changeSupport = new StatusChangeSupport();
    }
	
	// properties --------------------------------------------------------------

	public String getName() {
    	return name;
    }

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getRequirement() {
		return requirement;
	}
	
	public void setRequirement(String requirement) {
		this.requirement = requirement;
	}
	
	public String getTable() {
    	return tableName;
    }
	
	public void setTable(String tableName) {
		this.tableName = tableName;
	}

	public Intervals<VersionNumber> getAffectedVersions() {
		return affectedVersions;
	}
	
	public void setAffectedVersions(
			Intervals<VersionNumber> affectedVersions) {
		this.affectedVersions = affectedVersions;
	}
	
	public String getDefectType() {
		return defectType;
	}

	public void setDefectType(String defectType) {
		this.defectType = defectType;
	}
	
	public File getSourceFile() {
		return sourceFile;
	}
	
	public String getAuthor() {
		return author;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public File getReportFolder() {
	    return reportFolder;
    }

	public File getDocPage() {
		return docPage;
	}
	
	public File getDefectsOrErrorPage() {
		return defectsPage;
	}
	
	public File getTmpResultFile() {
    	return tmpResultFile;
    }

	public SanityCheckVerdict getVerdict() {
		return verdict;
	}
	
	public void setVerdict(SanityCheckVerdict verdict) {
		this.verdict = verdict;
	}

	public int getExecutionTime() {
	    return executionTime;
    }

	public void setExecutionTime(int executionTime) {
		this.executionTime = executionTime;
	}
	
	public String getExecutionNote() {
		return executionNote;
	}

	public void setExecutionNote(String executionNote) {
		this.executionNote = executionNote;
	}
	
	public Set<String> getTags() {
		return tags;
	}
	
	public void setTags(Set<String> tags) {
		this.tags = new HashSet<String>(tags);
	}
	
	public void addStatusListener(StatusChangeListener listener) {
		changeSupport.addListener(listener);
	}

	public void removeStatusListener(StatusChangeListener listener) {
		changeSupport.removeListener(listener);
	}
	
	protected void fireStatusChanged() {
		changeSupport.fireStatusChanged(this);
	}
	
	// functional interface ----------------------------------------------------
	
	public void accept(Visitor<Reportable> visitor) {
		visitor.visit(this);
	}
	
	public void checkApplicability(VersionNumber appVersion, Set<String> tablesToCheck, Set<String> tags) {
		SanityCheckVerdict verdict = SanityCheckVerdict.PENDING;
		if (!affectsVersion(appVersion))
			verdict = SanityCheckVerdict.NOT_APPLICABLE;
		else {
			if (!matchesTableFilter(tablesToCheck)) {
				this.executionNote = "Table filtered";
				verdict = SanityCheckVerdict.SKIPPED;
			} else if (!matchesTagFilter(tags)) {
				this.executionNote = "Tags filtered";
				verdict = SanityCheckVerdict.SKIPPED;
			}
		}
		setVerdict(verdict);
		fireStatusChanged();
	}

	public abstract String getDetails();

	public abstract void reset();

	public abstract DefectIterator perform(CheckContext context);
	
	public abstract long getDefectCount();
	
	public abstract void serialize(SimpleXMLWriter writer) throws SAXException;
	
	public boolean matchesTableFilter(Set<String> tablesToCheck) {
		if (CollectionUtil.isEmpty(tablesToCheck))
			return true; // no filter set
		return (this.tableName != null && tablesToCheck.contains(this.tableName));
	}

	public boolean matchesTagFilter(Set<String> requestedTags) {
		if (CollectionUtil.isEmpty(requestedTags))
			return true; // no filter set
		for (String tag : requestedTags)
			if (this.tags.contains(tag))
				return true;
		return false;
	}

	// java.lang.Object overrides ----------------------------------------------
	
	@Override
	public String toString() {
	    return name;
	}
	
	@Override
    public int hashCode() {
	    return name.hashCode();
    }

	@Override
    public boolean equals(Object obj) {
	    if (this == obj)
		    return true;
	    if (obj == null || getClass() != obj.getClass())
		    return false;
	    SanityCheck that = (SanityCheck) obj;
	    return this.name.equals(that.name);
    }

	public boolean affectsVersion(VersionNumber appVersion) {
		return (appVersion == null || affectedVersions.contains(appVersion));
	}

}
