/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.util.HashMap;
import java.util.Map;

import org.databene.commons.StringUtil;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.jdbacl.model.DBTable;
import org.xml.sax.SAXException;

/**
 * Verifies that an expression provides strings of a given minimum and maximum length.<br/><br/>
 * Created: 05.11.2011 06:31:05
 * @since 0.9.1
 * @author Volker Bergmann
 */
public class StringLengthQueryStrategy extends DefectQueryStrategy {
	
	private String tableName;
	private String reportColumnsSpec;
	private String expression;
	private String condition;
	private Integer min;
	private Integer max;

	public StringLengthQueryStrategy(
			String tableName, String reportColumnsSpec, String expression, String condition, Integer min, Integer max) {
		super(null, "stringLength");
		this.tableName = tableName;
		this.reportColumnsSpec = reportColumnsSpec;
		this.expression = expression;
		this.condition = condition;
		this.min = min;
		this.max = max;
	}

	@Override
	public DefectIterator perform(CheckContext context) {
		DBTable table = context.getDatabase().getTable(tableName);
		String reportColumnsList = renderReportColumns(this.reportColumnsSpec, table);
		StringBuilder builder = new StringBuilder("SELECT ").append(reportColumnsList);
		builder.append(" FROM ").append(table).append(" WHERE NOT ");
		builder.append("LENGTH(").append(expression).append(")");
		if (min != null && max != null)
			builder.append(" BETWEEN ").append(min).append(" AND ").append(max);
		else if (min != null)
			builder.append(" >= ").append(min);
		else
			builder.append(" <= ").append(max);
		if (!StringUtil.isEmpty(condition))
			builder.append(" AND ").append(condition);
		this.query = builder.toString();
		return super.perform(context);
	}
	
	protected String[] defaultReportColumns(DBTable table) {
		String[] columns = super.defaultReportColumns(table);
		return appendColumnIfNotIncluded(expression, columns);
	}
	
	@Override
	public void serialize(SimpleXMLWriter writer) throws SAXException {
		Map<String, String> atts = new HashMap<String, String>();
		atts.put("expression", expression);
		atts.put("reportColumnsSpec", reportColumnsSpec);
		atts.put("condition", condition);
		if (min != null)
			atts.put("min", String.valueOf(min));
		if (max != null)
			atts.put("max", String.valueOf(max));
		writer.startElement("stringLength", atts);
		writer.endElement("stringLength");
	}
	
	@Override
	public String toString() {
		String minString = (min != null ? min.toString() : "0");
		String maxString = (max != null ? max.toString() : "");
		return "stringLength[" + tableName + '.' + expression + " {" + minString + "," + maxString +"}]";
	}
	
}
