/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Map.Entry;

import org.databene.commons.Filter;
import org.databene.commons.OrderedMap;
import org.databene.commons.Visitor;
import org.databene.commons.collection.OrderedNameMap;
import org.databene.commons.version.VersionNumber;
import org.databene.dbsanity.Reporter;

/**
 * Default implementation of the {@link SuiteHolder} interface.<br/><br/>
 * Created: 13.11.2011 06:53:01
 * @since 0.9.2
 * @author Volker Bergmann
 */
public class SuiteHolderSupport extends AbstractSanityCheckSuite implements SuiteHolder {

	protected OrderedMap<String, SanityCheckSuite> childSuites;
	private Date startDate;

	public SuiteHolderSupport(String name, File reportFolder, SuiteHolder parent) {
		super(name, reportFolder, parent);
		this.childSuites = new OrderedNameMap<SanityCheckSuite>();
	}
	
	public void addChildSuite(SanityCheckSuite component) {
		this.childSuites.put(component.getName(), component);
	}
	
	public List<SanityCheckSuite> getChildSuites() {
	    return childSuites.values();
    }

	@SuppressWarnings("unchecked")
    public <T extends SanityCheckSuite> List<T> getChildSuites(Class<T> suiteType) {
		List<T> result = new ArrayList<T>();
		for (Entry<String, SanityCheckSuite> entry : childSuites.entrySet())
			if (suiteType.isAssignableFrom((entry.getValue().getClass())))
				result.add((T) entry.getValue());
	    return result;
    }

	public int countChecks() {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.countChecks();
		return count;
	}

	public void accept(Visitor<Reportable> visitor) {
		visitor.visit(this);
		for (SanityCheckSuite childSuite : childSuites.values())
			childSuite.accept(visitor);
	}

	public void checkApplicability(VersionNumber appVersion, Set<String> tablesToCheck, Set<String> tags) {
		for (SanityCheckSuite childSuite : getChildSuites())
			childSuite.checkApplicability(appVersion, tablesToCheck, tags);
	}

    public void perform(CheckContext checkContext, Reporter reporter) {
		startDate = new Date();
		Iterator<SanityCheckSuite> iterator = childSuites.values().iterator();
		while (iterator.hasNext())
			iterator.next().perform(checkContext, reporter);
	}
	
	public void reset() {
		for (SanityCheckSuite childSuite : childSuites.values())
			childSuite.reset();
	}
	
	public Date getStartDate() {
		return startDate;
	}
	
	public int countChecksWithVerdict(SanityCheckVerdict verdict) {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.countChecksWithVerdict(verdict);
		return count;
	}

	public int countChecks(Filter<SanityCheck> filter) {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.countChecks(filter);
		return count;
	}

    public long getDefectCount() {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.getDefectCount();
		return count;
	}

    public int getExecutionTime() {
		int sum = 0; 
		for (SanityCheckSuite childSuite : getChildSuites())
			sum += childSuite.getExecutionTime();
	    return sum;
    }

	@Override
	public String toString() {
		return name;
	}

}
