/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.lang.reflect.Method;

import javax.validation.ConstraintValidator;

import org.databene.commons.ArrayFormat;
import org.databene.commons.BeanUtil;
import org.databene.commons.ConfigurationError;
import org.databene.commons.Converter;
import org.databene.commons.StringUtil;
import org.databene.commons.Validator;
import org.databene.commons.converter.ArrayElementExtractor;
import org.databene.commons.converter.ArrayTypeConverter;
import org.databene.commons.validator.bean.BeanConstraintValidator;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.model.defect.TabularDefectIteratorProxy;
import org.databene.jdbacl.ArrayResultSetIterator;
import org.databene.jdbacl.model.DBTable;
import org.xml.sax.SAXException;

/**
 * {@link CheckStrategy} that uses a {@link Validator} to check database data on the client side.<br/><br/>
 * Created: 26.10.2011 20:58:03
 * @since 0.9.0
 * @author Volker Bergmann
 */
@SuppressWarnings({"rawtypes", "unchecked"})
public class ValidatorStrategy extends AbstractCheckStrategy {

	private Validator validator;
	private String tableName;
	private String[] checkedColumns;
	private String condition;
	
	private String reportColumnsSpec;
	private String[] reportColumns;
	
	private int[] checkedColumnIndexes;
	private long defectCount;
	
	public ValidatorStrategy(Object validatorObject, String tableName, String[] checkedColumns, String condition, String reportColumnsSpec) {
		this(validatorObject, tableName, checkedColumns, condition, reportColumnsSpec, null);
	}
	
	public ValidatorStrategy(Object validatorObject, String tableName, String[] checkedColumns, String condition, String[] reportColumns) {
		this(validatorObject, tableName, checkedColumns, condition, null, reportColumns);
	}
	
	private ValidatorStrategy(Object validatorObject, String tableName, String[] checkedColumns, String condition, String reportColumnsSpec, String[] reportColumns) {
		super("validator");
		if (validatorObject instanceof Validator)
			this.validator = (Validator) validatorObject;
		else if (validatorObject instanceof ConstraintValidator)
			this.validator = new BeanConstraintValidator((ConstraintValidator) validatorObject);
		else
			throw new ConfigurationError("Not a supported validator type: " + validatorObject.getClass().getName());
		this.tableName = tableName;
		this.checkedColumns = checkedColumns;
		this.condition = condition;
		this.reportColumnsSpec = reportColumnsSpec;
		this.reportColumns = reportColumns;
		this.defectCount = 0;
	}
	
	public String getDetails() {
		return validator.toString();
	}

	public Validator getValidator() {
		return validator;
	}

	public DefectIterator perform(CheckContext context) {
		DBTable table = context.getDatabase().getTable(tableName);
		if (reportColumns == null)
			reportColumns = parseReportColumns(reportColumnsSpec, table);
		appendColumnsIfNotIncluded(checkedColumns, reportColumns);
		this.checkedColumnIndexes = new int[checkedColumns.length];
		for (int i = 0; i < checkedColumns.length; i++)
			this.checkedColumnIndexes[i] = StringUtil.indexOfIgnoreCase(checkedColumns[i], reportColumns);
		String query = "SELECT " + ArrayFormat.format(reportColumns) + " FROM " + table;
		if (!StringUtil.isEmpty(condition))
			query += " WHERE " + condition;
		ArrayResultSetIterator iterator = new ArrayResultSetIterator(context.getConnection(), query);
		return new Proxy(iterator);
	}
	
	@Override
	protected String[] defaultReportColumns(DBTable table) {
		return appendColumnsIfNotIncluded(checkedColumns, super.defaultReportColumns(table));
	}
	
	public long getDefectCount() {
		return defectCount;
	}

	public void reset() {
		defectCount = 0;
	}

	@Override
	public String toString() {
		return tableName + '[' + ArrayFormat.format(checkedColumns) + "] matches " + validator.toString();
	}

	public class Proxy extends TabularDefectIteratorProxy {
		
		private DefectRow next;
		private Converter argumentConverter;
		
		public Proxy(ArrayResultSetIterator source) {
			super(source);
			Class<? extends Validator> validatorClass = validator.getClass();
			checkValidatorType(validatorClass);
			fetchNext();
		}

		public boolean hasNext() {
			return (next != null);
		}

		public DefectRow next() {
			DefectRow result = next;
			fetchNext();
			return result;
		}

		private void checkValidatorType(Class<? extends Validator> validatorClass) {
			// TODO v1.0 it would be better to fetch the generic type Validator<E> from the class by reflection
			Method[] validationCandidates = BeanUtil.findMethodsByName(validatorClass, "valid");
			Method validationMethod = null;
			for (Method candidate : validationCandidates)
				if (candidate.getParameterTypes().length == 1) {
					if (validationMethod == null || validationMethod.getParameterTypes()[0] == Object.class)
						validationMethod = candidate;
					else if (candidate.getParameterTypes()[0] != Object.class)
						throw new ConfigurationError("Several validate() methods with a single argument are defined in " + validatorClass);
				}
			if (validationMethod == null)
				throw new ConfigurationError("No validate() method with a single argument is defined in " + validatorClass);
			Class validatedType = validationMethod.getParameterTypes()[0];
			if (!validatedType.isArray() && checkedColumnIndexes.length != 1)
				throw new ConfigurationError("The number of checked columns needs to be 1 but was " + checkedColumnIndexes.length);
			if (validatedType.isArray())
				argumentConverter = new ArrayTypeConverter(validatedType.getComponentType());
			else
				argumentConverter = new ArrayElementExtractor(Object.class, 0);

		}

		private void fetchNext() {
			DefectRow tmp = null;
			boolean valid = true;
			while (super.hasNext() && valid) {
				tmp = super.next();
				Object[] args = new Object[checkedColumnIndexes.length];
				for (int i = 0; i < checkedColumnIndexes.length; i++)
					args[i] = tmp.get(checkedColumnIndexes[i]);
				Object convertedArgs = argumentConverter.convert(args);
				valid = validator.valid(convertedArgs);
			}
			if (!valid) {
				next = tmp;
				defectCount++;
			} else
				next = null;
		}

	}

	public void serialize(SimpleXMLWriter writer) throws SAXException {
		throw new UnsupportedOperationException("CheckStrategy.serialize() is not implemented"); // TODO v0.9.5 implement CheckStrategy.serialize
	}

}
