/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.data;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.databene.commons.ArrayFormat;
import org.databene.commons.ArrayUtil;
import org.databene.commons.CollectionUtil;
import org.databene.commons.ConfigurationError;
import org.databene.commons.ParseUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.jdbacl.model.DBTable;

/**
 * {@link Testee} implementation which iterates over the rows of a table.<br/><br/>
 * Created: 25.01.2012 12:16:22
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class ColumnTestee extends Testee {
	
	private String tableName;
	private String columnsSpec;
	private String identityColumnsSpec;
	private String reportColumnsSpec;
	private String condition;
	
	private String query;
	private int[] identityColumnIndexes;
	private int[] checkedColumnIndexes;
	private String[] columnNames;
	
	// construction and initialization ---------------------------------------------------------------------------------
	
	public ColumnTestee(String tableName, String columnsSpec, 
			String identityColumnsSpec, String reportColumnsSpec, String condition) {
		this.tableName = tableName;
		this.columnsSpec = columnsSpec;
		this.identityColumnsSpec = identityColumnsSpec;
		this.reportColumnsSpec = reportColumnsSpec;
		this.condition = condition;
		this.query = null;
		this.checkedColumnIndexes = null;
		this.identityColumnIndexes = null;
	}

	protected void onInit() {
		DBTable table = context.getDatabase().getTable(tableName);
		int indexBase = 0;
		String[] specifiedCols = StringUtil.splitAndTrim(columnsSpec, ',');
		List<String> cols = new ArrayList<String>();
		
		// add a column and offset for each PK column not in colSpec
		for (String pkColumn : table.getPKColumnNames()) {
			if (!ArrayUtil.contains(pkColumn, specifiedCols)) {
				cols.add(pkColumn.toUpperCase());
				indexBase++;
			}
		}
		
		// find out the indexes of the identity columns
		String[] idColumnTokens = null;
		if (!StringUtil.isEmpty(identityColumnsSpec)) {
			idColumnTokens = StringUtil.splitAndTrim(identityColumnsSpec, ',');
			this.identityColumnIndexes = new int[idColumnTokens.length];
			for (int i = 0; i < idColumnTokens.length; i++) {
				String idColumnName = idColumnTokens[i];
				if (ParseUtil.isNonNegativeNumber(idColumnName))
					idColumnName = specifiedCols[Integer.parseInt(idColumnName)];
				int index = StringUtil.indexOfIgnoreCase(idColumnName, specifiedCols);
				if (index >= 0)
					this.identityColumnIndexes[i] = index;
				else
					throw new SyntaxError("Identity column '" + idColumnName + "' is not part of the columns list to check", columnsSpec);
			}
		}
		
		// add all columns in colSpec and declare their index
		checkedColumnIndexes = new int[specifiedCols.length];
		for (int i = 0; i < specifiedCols.length; i++) {
			cols.add(specifiedCols[i].toUpperCase());
			checkedColumnIndexes[i] = indexBase + i;
		}
		
		// add all columns in reportColumnsSpec which have not been used yet
		if (!StringUtil.isEmpty(reportColumnsSpec)) {
			String[] reportColumns = StringUtil.trimAll((String[]) ArrayFormat.parse(reportColumnsSpec, ",", String.class));
			for (String reportColumn : reportColumns) {
				reportColumn = reportColumn.toUpperCase();
				if (!cols.contains(reportColumn))
					cols.add(reportColumn);
			}
		}
		
		// store column names
		this.columnNames = CollectionUtil.toArray(cols);
		
		// calculate query
		this.query = "SELECT " + CollectionUtil.formatCommaSeparatedList(cols, null) + " FROM " + tableName;
		if (!StringUtil.isEmpty(condition))
			this.query += " WHERE " + condition;
	}

	// Testee interface implementation ---------------------------------------------------------------------------------
	
	public String getQuery() {
		return (isInitialized() ? query : context.localize("initialization_failed"));
	}
	
	public String[] getColumnNames() {
		assertInitialized();
		return columnNames;
	}
	
	public int[] getIdentityColumnIndexes() {
		return identityColumnIndexes;
	}
	
	public int[] getCheckedColumnIndexes() {
		assertInitialized();
		return checkedColumnIndexes;
	}

	public TesteeSampleIterator iterator() {
		assertInitialized();
		try {
			return new TesteeSampleIterator(this, context.getConnection());
		} catch (SQLException e) {
			throw new ConfigurationError("Error executing " + this, e);
		}
	}
	
	// java.lang.Object overrides --------------------------------------------------------------------------------------

	@Override
	public String toString() {
		return getClass().getSimpleName() + "[table=" + tableName + ", columns=[" + columnsSpec + "]]";
	}

}
