/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.data;

import java.util.Collection;

import org.databene.commons.ConfigurationError;
import org.databene.commons.HeavyweightIterable;
import org.databene.commons.HeavyweightIterator;
import org.databene.commons.IOUtil;
import org.databene.commons.MathUtil;
import org.databene.commons.bean.ArrayWithIdentity;
import org.databene.commons.collection.MarkedMap;
import org.databene.commons.converter.ToStringConverter;
import org.databene.dbsanity.model.CheckContext;
import org.databene.dbsanity.model.DefectRow;
import org.databene.script.math.ArithmeticEngine;

/**
 * Common parent class for all components that implements data matching.<br/><br/>
 * Created: 25.01.2012 16:13:48
 * @since 0.9.3
 * @author Volker Bergmann
 */
public abstract class DataMatcher {
	
	private int[] identityColumnIndexes;
	private int maxIdentityColumn;
	private CheckContext context;
	private MarkedMap<ArrayWithIdentity, Object[]> referenceSamples;
	
	public DataMatcher(HeavyweightIterable<Object[]> reference, int[] identityColumnIndexes, CheckContext context) {
		this.identityColumnIndexes = identityColumnIndexes;
		this.maxIdentityColumn = (identityColumnIndexes != null ? MathUtil.max(identityColumnIndexes) : -1);
		this.context = context;
		// iterate through all reference data
		HeavyweightIterator<Object[]> referenceIterator = reference.iterator();
		while (referenceIterator.hasNext())
			addReferenceSample(referenceIterator.next());
		IOUtil.close(referenceIterator);
	}
	
	protected void addReferenceSample(Object[] referenceSample) {
		if (this.referenceSamples == null)
			this.referenceSamples = new MarkedMap<ArrayWithIdentity, Object[]>();
		if (maxIdentityColumn >= 0 && referenceSample.length < maxIdentityColumn + 1)
			throw new ConfigurationError("Number of reference columns does not match the testee");
		for (int i = 0; i < referenceSample.length; i++)
			referenceSample[i] = ToStringConverter.convert(referenceSample[i], null);
		ArrayWithIdentity id = IdentityUtil.identity(referenceSample, identityColumnIndexes);
		this.referenceSamples.put(id, referenceSample);
	}
	
	protected Object[] findAndMarkReferenceSampleByIdentity(TesteeSample testeeSample) {
		if (referenceSamples.size() == 0)
			return null;
		ArrayWithIdentity id = testeeSample.getId();
		if (this.identityColumnIndexes != null && id.getElementCount() != this.identityColumnIndexes.length)
			throw new ConfigurationError("The column count of reference (" + this.identityColumnIndexes.length + ") " +
					"does not match the testee (" + id.getElementCount() + ")");
		Object[] sample = referenceSamples.get(id);
		if (sample != null)
			referenceSamples.mark(id);
		return sample;
	}
	
	public Collection<Object[]> getUnmarkedReferenceSamples() {
		return referenceSamples.unmarkedEntries().values();
	}
	
	public abstract DefectRow check(TesteeSample testeeSample);
	
	public abstract Collection<Object[]> getMissingSamples();
	
	protected static boolean equalCheckedContent(Object[] referenceSample, TesteeSample testeeSample) {
		if (testeeSample == null)
			return (referenceSample == null);
		if (referenceSample == null)
			return false;
		Object[] checkedTesteeCells = testeeSample.extractCheckedElements();
		int length = checkedTesteeCells.length;
		if (length != referenceSample.length)
			return false;
		for (int i = 0; i < length; i++) {
			if (!equalValues(referenceSample[i], checkedTesteeCells[i]))
				return false;
		}
		return true;
	}

	protected DefectRow compare(Object[] refSample, TesteeSample testeeSample) {
		if (refSample == null)
			return new DefectRow(testeeSample.getCells()).add(illegalSampleMessage());
		else if (testeeSample == null)
			return new DefectRow(refSample).add(missingSampleMessage());
		else {
			boolean differs = false;
			DefectRow result = new DefectRow(testeeSample.getCells());
			Object[] testeeCells = testeeSample.getCells();
			int headingUncheckedColumnCount = testeeSample.getHeadingUncheckedColumnCount();
			for (int i = 0; i < refSample.length; i++) {
				if (!equalValues(refSample[i], testeeCells[i + headingUncheckedColumnCount])) {
					String value = String.valueOf(result.get(i));
					value += " [" + refSample[i] + "]";
					result.set(i, value, true);
					differs = true;
				}
			}
			if (!differs)
				return null;
			result.add(inconsistenSampleMessage(), true);
			return result;
		}
	}

	private static boolean equalValues(Object value1, Object value2) {
		return ArithmeticEngine.defaultInstance().equals(value1, value2);
	}
	
	public String illegalSampleMessage() {
		return context.localize("data_entry_not_allowed");
	}

	public String missingSampleMessage() {
		return context.localize("data_entry_missing");
	}

	public String inconsistenSampleMessage() {
		return context.localize("data_entry_misconfigured");
	}

}
