/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.data;

import java.util.Iterator;

import org.databene.commons.HeavyweightIterable;
import org.databene.commons.ProgrammerError;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.model.AbstractCheckStrategy;
import org.databene.dbsanity.model.CheckContext;
import org.databene.dbsanity.model.CheckStrategy;
import org.databene.dbsanity.model.DefectIterator;
import org.databene.dbsanity.model.DefectRow;
import org.databene.dbsanity.model.defect.AbstractDefectIterator;
import org.xml.sax.SAXException;

/**
 * {@link CheckStrategy} implementation for comparison of multi-column data sets.<br/><br/>
 * Created: 24.01.2012 18:30:27
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class DataStrategy extends AbstractCheckStrategy {

	private String match;
	private Testee testee;
	private HeavyweightIterable<Object[]> reference;
	CheckContext context;
	private int defectCount;
	
	public DataStrategy(String match, Testee testee, HeavyweightIterable<Object[]> reference) {
		super("data");
		this.match = match;
		this.testee = testee;
		this.reference = reference;
	}

	public String getDetails() {
		return getQuery();
	}

	public String getQuery() {
		return testee.getQuery();
	}

	public DefectIterator perform(CheckContext context) {
		this.context = context;
		defectCount = 0;
		testee.init(context);
		int[] idColumnIndexes = testee.getIdentityColumnIndexes();
		
		DataMatcher matcher;
		if ("superset".equals(match))
			matcher = new SupersetMatcher(reference, idColumnIndexes, context);
		else if ("subset".equals(match))
			matcher = new SubsetMatcher(reference, idColumnIndexes, context);
		else if ("set".equals(match))
			matcher = new SetMatcher(reference, idColumnIndexes, context);
		else
			throw new ProgrammerError("Not a supported match type. This should have been caught earlier");
		
		// iterate and check testee
		return new DataDefectIterator(testee, matcher);
	}

	public long getDefectCount() {
		return defectCount;
	}

	public void reset() {
		this.defectCount = 0;
	}
	
	protected String[] defineColumnLabels(Testee testee) {
		String[] testeeColumnNames = testee.getColumnNames();
		String[] columnLabels = new String[testeeColumnNames.length + 1];
		System.arraycopy(testeeColumnNames, 0, columnLabels, 0, testeeColumnNames.length);
		columnLabels[columnLabels.length - 1] = context.localize("error_message");
		return columnLabels;
	}

	class DataDefectIterator extends AbstractDefectIterator {
		
		private int[] checkedColumnIndexes;
		private TesteeSampleIterator testeeIterator;
		private DataMatcher matcher;
		
		private Iterator<Object[]> collectedErrorIterator;
		private boolean hasNext;
		private DefectRow next;
		
		public DataDefectIterator(Testee testee, DataMatcher matcher) {
			super(defineColumnLabels(testee));
			this.testeeIterator = testee.iterator();
			this.checkedColumnIndexes = testee.getCheckedColumnIndexes();
			this.matcher = matcher;
			fetchNext();
		}
		
		public boolean hasNext() {
			return hasNext;
		}

		public DefectRow next() {
			defectCount++;
			DefectRow result = next;
			fetchNext();
			return result;
		}

		private void fetchNext() {
			while (testeeIterator.hasNext()) {
				TesteeSample testeeSample = testeeIterator.next();
				DefectRow defectRow = matcher.check(testeeSample);
				if (defectRow != null) {
					this.hasNext = true;
					this.next = defectRow;
					return;
				}
			}
			// when arriving here, all testee elements have been iterated
			// so it is time to iterate the collected errors
			if (collectedErrorIterator == null)
				collectedErrorIterator = matcher.getMissingSamples().iterator();
			if (collectedErrorIterator.hasNext()) {
				this.hasNext = true;
				this.next = createMissingSampleRow(collectedErrorIterator.next());
			} else {
				this.hasNext = false;
				this.next = null;
			}
		}

		private DefectRow createMissingSampleRow(Object[] sample) {
			int columnCount = getColumnNames().length;
			DefectRow row = new DefectRow(columnCount);
			for (int i = 0; i < checkedColumnIndexes.length; i++)
				row.set(checkedColumnIndexes[i], sample[i]);
			row.set(columnCount - 1, matcher.missingSampleMessage(), true);
			return row;
		}

	}

	public void serialize(SimpleXMLWriter writer) throws SAXException {
		throw new UnsupportedOperationException("CheckStrategy.serialize() is not implemented"); // TODO v0.9.5 implement CheckStrategy.serialize
	}
	
}
