/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.io.File;
import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.ConfigurationError;
import org.databene.commons.Converter;
import org.databene.commons.Encodings;
import org.databene.commons.HeavyweightIterable;
import org.databene.commons.IOUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.converter.ArrayTypeConverter;
import org.databene.commons.iterator.HeadedTabularIterable;
import org.databene.commons.iterator.SelectiveTabularIterable;
import org.databene.commons.iterator.TabularIterable;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.databene.dbsanity.model.data.ColumnTestee;
import org.databene.dbsanity.model.data.DataStrategy;
import org.databene.dbsanity.model.data.Testee;
import org.databene.document.csv.CSVSource;
import org.databene.document.xls.XLSLineSource;
import org.databene.webdecs.DataSource;
import org.databene.webdecs.util.ConvertingDataSource;
import org.databene.webdecs.util.JavaIterableFromDataSource;
import org.databene.webdecs.util.OffsetDataSource;
import org.w3c.dom.Element;

/**
 * Parses &lt;data&gt; elements into {@link DataStrategy} instances.<br/><br/>
 * Created: 24.01.2012 18:12:06
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class DataParser extends AbstractDbSanityXMLParser {
	
	private static final String IDENTITY_COLUMNS = "identityColumns";
	private static final String REFERENCE_IS = "referenceIs";
	private static final String SHEET = "sheet";
	private static final String FILE = "file";
	private static final String HEADER_OFFSET = "headerOffset";

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet(REFERENCE_IS);
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet();

	private static final String DEFAULT_DEFECT_TYPE = "data";

	//private static final String CHECKED_INDEXES = "checkedIndexes"; // TODO v0.9.x use when supporting <testee query>

	public DataParser() {
		super("data", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}

	@Override
	public DataStrategy parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
		StrategyBasedCheck parent = (StrategyBasedCheck) parent(parentPath);
		String tableName = getTableName(parent, XMLUtil.format(element));

		// parse matching type
		String match = getRequiredAttribute(REFERENCE_IS, element).trim();
		
		// parse child elements
		Testee testee = null;
		HeavyweightIterable<Object[]> reference = null;
		for (Element child : XMLUtil.getChildElements(element)) {
			String childName = child.getNodeName();
			if ("testee".equals(childName))
				testee = parseTestee(child, tableName);
			else if ("reference".equals(childName))
				reference = parseReference(child, parent.getSourceFile());
			else
				syntaxError("No a supported child element of <" + element.getNodeName() + ">", child);
		}
		if (testee == null)
			syntaxError("<" + element.getNodeName() + "> must have a <testee> child", element);
		if (reference == null)
			syntaxError("<" + element.getNodeName() + "> must have a <reference> child", element);
		
		DataStrategy strategy = new DataStrategy(match, testee, reference);
		parent.setStrategy(strategy);
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
	    return strategy;
	}

	private Testee parseTestee(Element element, String tableName) {
		checkSupportedAttributes(element, COLUMNS, REPORT_COLUMNS, CONDITION, IDENTITY_COLUMNS);
		String columnsSpec = getOptionalAttribute(COLUMNS, element);
		String identityColumns = getOptionalAttribute(IDENTITY_COLUMNS, element);
		// String query = getOptionalAttribute("query", element); // TODO v0.9.x support query
		if (!StringUtil.isEmpty(columnsSpec))
			return new ColumnTestee(tableName, columnsSpec, identityColumns, 
					getOptionalAttribute(REPORT_COLUMNS, element), getOptionalAttribute(CONDITION, element));
		//else if (!StringUtil.isEmpty(query)) // TODO v0.9.x support query
		//	return new QueryTestee(query, getOptionalAttribute(CHECKED_INDEXES, element));
		else
			throw new SyntaxError("Invalid <testee> spec: Need to specify columns or query", XMLUtil.format(element));
	}

	private HeavyweightIterable<Object[]> parseReference(Element element, File referrer) {
		checkSupportedAttributes(element, FILE, SHEET, COLUMNS, HEADER_OFFSET);
		String fileName = getOptionalAttribute(FILE, element);
		String sheetName = getOptionalAttribute(SHEET, element);
		Integer headerOffset = parseOptionalInteger(HEADER_OFFSET, element);
		if (headerOffset == null)
			headerOffset = 0;
		String columnsSpec = getOptionalAttribute(COLUMNS, element);
		if (!StringUtil.isEmpty(fileName))
			return createFileReference(fileName, sheetName, headerOffset, referrer, columnsSpec);
		else
			throw new SyntaxError("Invalid <reference> spec: Need to specify file", XMLUtil.format(element));
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private HeavyweightIterable<Object[]> createFileReference(
			String fileName, String sheetName, int headerOffset, File referrer, String columnsSpec) {
		fileName = IOUtil.resolveRelativeUri(fileName, referrer.getParent());
		if (!IOUtil.isURIAvailable(fileName))
			throw new ConfigurationError("File not found: " + fileName);
		DataSource<Object[]> dataSource;
		if (fileName.toLowerCase().endsWith(".xls"))
			dataSource = new XLSLineSource(fileName, sheetName);
		else if (fileName.toLowerCase().endsWith(".csv")) {
			CSVSource csvSource = new CSVSource(fileName, ',', Encodings.UTF_8, true, true);
			// CSVSource creates String[], so convert them to Object[]
			Converter converter = new ArrayTypeConverter<Object>(Object.class);
			dataSource = new ConvertingDataSource(csvSource, converter);
		} else
			throw new ConfigurationError("File type not supported: " + fileName);
		if (headerOffset > 0)
			dataSource = new OffsetDataSource<Object[]>(dataSource, headerOffset);
		JavaIterableFromDataSource<Object[]> iterable = new JavaIterableFromDataSource<Object[]>(dataSource);
		TabularIterable tabularIterable = new HeadedTabularIterable(iterable);
		String[] columns = null;
		if (!StringUtil.isEmpty(columnsSpec))
			columns = StringUtil.trimAll(columnsSpec.split(","));
		SelectiveTabularIterable selectiveTabularIterable = new SelectiveTabularIterable(tabularIterable, columns);
		return selectiveTabularIterable;
	}

}
