/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.NumberSizeQueryStrategy;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.w3c.dom.Element;

/**
 * Parses the &lt;numberSize&gt; part of a &lt;check&gt; and creates a SQL defect query.<br/><br/>
 * Created: 26.05.2011 12:39:04
 * @since 0.8
 * @author Volker Bergmann
 */
public class NumberSizeParser extends AbstractDbSanityXMLParser {

	private static final String TOTAL_DIGITS = "totalDigits";
	private static final String FRACTION_DIGITS = "fractionDigits";
	private static final String DEFAULT_DEFECT_TYPE = "numberSize";
	
	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet(EXPRESSION);
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(
			REPORT_COLUMNS, TOTAL_DIGITS, FRACTION_DIGITS, MIN, MAX);

	public NumberSizeParser() {
		super("numberSize", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
		StrategyBasedCheck parent = (StrategyBasedCheck) this.parent(parentPath);
		String tableName = getTableName(parent, XMLUtil.format(element));

		// parse XML element
		String expression = getOptionalAttribute(EXPRESSION, element);		
		String condition = getCondition(element);		
		String totalDigitsSpec = getOptionalAttribute(TOTAL_DIGITS, element);		
		String fractionDigitsSpec = getOptionalAttribute(FRACTION_DIGITS, element);
		String min = getOptionalAttribute("min", element);		
		String max = getOptionalAttribute("max", element);
		if (StringUtil.isEmpty(totalDigitsSpec) && StringUtil.isEmpty(fractionDigitsSpec) && 
				StringUtil.isEmpty(min) && StringUtil.isEmpty(max))
			throw new SyntaxError("'min', 'max', 'totalDigits' or 'fractionDigits' must be specified", 
					XMLUtil.format(element));

		Integer totalDigits = (totalDigitsSpec != null ? Integer.parseInt(totalDigitsSpec) : null);
		Integer fractionDigits = (fractionDigitsSpec != null ? Integer.parseInt(fractionDigitsSpec) : null);
		
	    String reportColumnsSpec = getReportColumnsSpec(element);
		DefectQueryStrategy strategy = new NumberSizeQueryStrategy(tableName, expression, condition, 
				totalDigits, fractionDigits, min, max, reportColumnsSpec);
		parent.setStrategy(strategy);
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
	    return strategy;
	}

}
