/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.CheckContext;
import org.databene.dbsanity.model.CheckStrategy;
import org.databene.dbsanity.model.DefectIterator;
import org.databene.dbsanity.model.DefectRow;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.databene.dbsanity.model.defect.DefectListIterator;
import org.databene.jdbacl.DBUtil;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * Verifies the number of rows in a database table.<br/><br/>
 * Created: 27.10.2011 08:17:09
 * @since 0.9.0
 * @author Volker Bergmann
 */
public class RowCountParser extends AbstractDbSanityXMLParser {

	private static final String MESSAGE = "message";
	private static final String DEFAULT_DEFECT_TYPE = "rowCount";

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet();
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet("min", "max", MESSAGE);

	public RowCountParser() {
		super("rowCount", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		String xmlDef = XMLUtil.format(element);

		// check parent
		StrategyBasedCheck parent = (StrategyBasedCheck) this.parent(parentPath);
		String table = parent.getTable();
		if (StringUtil.isEmpty(table))
			throw new SyntaxError("Missing 'table' spec in outer <check>", xmlDef);

		// parse XML element
		Long min   = parseOptionalLong("min", element);
		Long max   = parseOptionalLong("max", element);
		if (min == null && max == null)
			throw new SyntaxError("'min' and/or 'max' must be specified", xmlDef);
		if (min != null && max != null && min > max)
			throw new SyntaxError("min is configured to be greater than max", xmlDef);
		String messagePattern = getOptionalAttribute(MESSAGE, element);
		if (messagePattern == null)
			messagePattern = defaultMessagePattern(min, max);
		
		RowCountStrategy strategy = new RowCountStrategy(table, min, max, messagePattern, xmlDef);
	    parent.setStrategy(strategy);
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
	    return strategy;
	}

	private String defaultMessagePattern(Long min, Long max) {
		//{0} min, {1} max, {2} actual row count
		if (min != null && max != null) {
			if (min == max)
				return "Expected exactly {0} rows, found {2}";
			else
				return "Row count {2} is not in the expected range between {0} and {1}";
		}
		if (min != null)
			return "Row count {2} is below the required minimum of {0}";
		return "Row count {2} is above the required maximum of {1}";
	}

	public class RowCountStrategy implements CheckStrategy {

		private String table;
		private Long min;
		private Long max;
		private String messagePattern;
		private String details;
		private long defectCount;
		
		public RowCountStrategy(String table, Long min, Long max, String messagePattern, String details) {
			this.table = table;
			this.min = min;
			this.max = max;
			this.messagePattern = messagePattern;
			this.details = details;
			this.defectCount = 0;
		}

		public String getDetails() {
			return details;
		}

		public DefectIterator perform(CheckContext context) {
			long rowCount = DBUtil.queryLong("select count(*) from " + table, context.getConnection());
			DefectListIterator result = new DefectListIterator("Message");
			if (!checkRowCount(rowCount)) {
				String message = MessageFormat.format(messagePattern, min, max, rowCount);
				result.addDefect(new DefectRow(new Object[] { message }));
			}
			return result;
		}

		public long getDefectCount() {
			return defectCount;
		}

		private boolean checkRowCount(long actualRowCount) {
			if (min != null && actualRowCount < min) {
				defectCount = min - actualRowCount;
				return false;
			}
			if (max != null && actualRowCount > max) {
				defectCount = actualRowCount - max;
				return false;
			}
			return true;
		}

		public void reset() {
			defectCount = 0;
		}

		public void serialize(SimpleXMLWriter writer) throws SAXException {
			Map<String, String> atts = new HashMap<String, String>();
			if (min != null)
				atts.put("min", String.valueOf(min));
			if (max != null)
				atts.put("max", String.valueOf(max));
			atts.put("messagePattern", messagePattern);
			writer.startElement("rowCount", atts);
			writer.endElement("rowCount");
		}

		public String getDefectType() {
			return "rowCount";
		}

	}

}
