/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.databene.commons.ArrayUtil;
import org.databene.commons.FileUtil;
import org.databene.commons.IOUtil;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.DbSanity;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.model.SanityCheckFolder;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.w3c.dom.Document;

/**
 * Parses DB Sanity file hierarchies and provides them as a tree of {@link SanityCheckFolder}, 
 * {@link SanityCheckFile} and {@link SanityCheck} instances.<br/><br/>
 * Created: 13.10.2010 13:46:12
 * @since 1.0
 * @author Volker Bergmann
 */
public class SanityCheckSuiteParser {
	
	public SanityCheckSuiteParser() {
	}

	// interface ---------------------------------------------------------------

	public SanityCheckSuite parseHierarchy(File definitionFileOrFolder, File reportFolder, File tempFolder) throws IOException {
        DbSanityParseContext context = new DbSanityParseContext(reportFolder, tempFolder);
		return parseHierarchy(definitionFileOrFolder, context);
    }
	
	public SanityCheckSuite parseHierarchy(File definitionFileOrFolder, DbSanityParseContext context) throws IOException {
		return parseSuite(definitionFileOrFolder, context, new Object[0]);
    }
	
	// private helper methods --------------------------------------------------

	private SanityCheckSuite parseSuite(File definitionFileOrFolder, DbSanityParseContext context, Object[] parentPath) 
				throws IOException {
		// check preconditions
		if (!definitionFileOrFolder.exists())
			throw new RuntimeException("Test definition not found: " + definitionFileOrFolder);
		
    	SanityCheckFolder parent = AbstractDbSanityXMLParser.getParentSanityCheckFolder(parentPath);
		SanityCheckSuite result;
	    if (definitionFileOrFolder.isDirectory()) {
	    	if (parent == null) {
	    		// when parsing the suite's root folder, simply parse each file
    			result = parseFolder(definitionFileOrFolder, context, parentPath);
	    	} else {
	    		// when parsing a sub folder, first create a sub context...
	    		File tempFolder = FileUtil.fileOfLimitedPathLength(context.getTempFolder(), 
	    				definitionFileOrFolder.getName(), ".result", DbSanity.MAX_PATH_LENGTH, false);
	    		File reportFolder = new File(context.getReportFolder(), definitionFileOrFolder.getName());
	    		DbSanityParseContext subContext = context.createSubContext(
	    				definitionFileOrFolder, reportFolder, tempFolder);
	    		// ...then parse the files therein
	    		result = parseFolder(definitionFileOrFolder, subContext, parentPath);
	    	}
	    } else if (definitionFileOrFolder.isFile()) {
	    	if (parent == null) {
	    		// if the whole check suite consists of a single file, simply parse it
    			result = parseFile(definitionFileOrFolder, context, parentPath);
	    	} else {
	    		// when parsing a sub folder, first create a sub context... 
		    	String subFolderName = cutSuffix(definitionFileOrFolder.getName());
		    	File subReportFolder = new File(context.getReportFolder(), subFolderName);
		    	File subTempFolder = new File(context.getTempFolder(), subFolderName);
		    	DbSanityParseContext subContext = context.createSubContext(
		    			definitionFileOrFolder, subReportFolder, subTempFolder);
		    	// ...then parse the file with this sub context
				result = parseFile(definitionFileOrFolder, subContext, parentPath);
	    	}
	    } else
			throw new RuntimeException(
					"Test definition folder is neither a Directory nor a File: " + definitionFileOrFolder);
	    return result;
    }

	static String cutSuffix(String fileName) {
		String result = fileName.trim();
		String lcFileName = result.toLowerCase();
		if (lcFileName.endsWith(".dbs.xml"))
			result = result.substring(0, result.length() - ".dbs.xml".length());
		else if (lcFileName.endsWith(".xml"))
			result = result.substring(0, result.length() - ".xml".length());
		return result;
    }

	private SanityCheckFolder parseFolder(File sourceFolder, DbSanityParseContext context, Object[] parentPath) throws IOException {
		String name = sourceFolder.getName();
		SanityCheckFolder result = new SanityCheckFolder(name, sourceFolder, context.getReportFolder(), 
				AbstractDbSanityXMLParser.getParentSanityCheckFolder(parentPath));
		for (File subFile : listSuites(sourceFolder)) {
			Object[] subPath = ArrayUtil.append(result, parentPath);
			SanityCheckSuite subSuite = parseSuite(subFile, context, subPath);
			if (subSuite != null) {
				// add the suite if the file was not dbsanity.xml
				result.addChildSuite(subSuite);
			}
		}
	    return result;
    }

	private List<File> listSuites(File definitionFolder) {
		List<File> result = new ArrayList<File>();
	    for (File file : definitionFolder.listFiles())
	    	if (containsCheck(file))
	    		result.add(file);
	    return result;
    }

	private boolean containsCheck(File fileOrDirectory) {
    	if (fileOrDirectory.isFile())
    		return fileOrDirectory.getName().toLowerCase().endsWith(".xml");
    	else {
		    File[] children = fileOrDirectory.listFiles();
		    if (children == null)
		    	return false;
			for (File child : children) {
		    	if (containsCheck(child))
		    		return true;
		    }
			return false;
    	}
	}

	private SanityCheckFile parseFile(File sourceFile, DbSanityParseContext context, Object[] parentPath) 
			throws IOException {
		if ("dbsanity.xml".equals(sourceFile.getName().toLowerCase())) {
			return null; // TODO v1.0 read config here and put it into context?
		} else {
	        String xmlString = IOUtil.getContentOfURI(sourceFile.getCanonicalPath());
	        Document doc = XMLUtil.parseString(xmlString);
	        DbSanityParseContext subContext = context.createSubContext(
	        		sourceFile, context.getReportFolder(), context.getTempFolder());
			return (SanityCheckFile) subContext.parseElement(doc.getDocumentElement(), parentPath);
		}
    }

}
