/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.databene.dbsanity.model.StringLengthQueryStrategy;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;

/**
 * Parses the &lt;stringLength&gt; part of a &lt;check&gt; and creates a SQL defect query.<br/><br/>
 * Created: 26.05.2011 12:44:57
 * @since 0.8
 * @author Volker Bergmann
 */
public class StringLengthParser extends AbstractDbSanityXMLParser {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(StringLengthParser.class);

	private static final String DEFAULT_DEFECT_TYPE = "stringLength";

	private static final String MAX_LENGTH = "maxLength";
	private static final String MIN_LENGTH = "minLength";
	
	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet(EXPRESSION);
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(
			REPORT_COLUMNS, MIN, MAX, MIN_LENGTH, MAX_LENGTH);
	
	public StringLengthParser() {
		super("stringLength", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
		StrategyBasedCheck parent = (StrategyBasedCheck) this.parent(parentPath);
		String tableName = parent.getTable();

		// parse expression
		String expression = getRequiredAttribute(EXPRESSION, element).trim();
		String condition = getCondition(element);
		
		// parse min
		String minLengthSpec = getOptionalAttribute(MIN_LENGTH, element);
		if (minLengthSpec != null)
			LOGGER.warn("Attribute 'minLength' of <stringLength> has been renamed to 'min'. " +
					"Please update your check definition");
		String minSpec = getOptionalAttribute(MIN, element);
		Integer min = (minSpec != null ? Integer.parseInt(minSpec) : null);
		if (min == null && minLengthSpec != null)
			min = Integer.parseInt(minLengthSpec);
		
		// parse max
		String maxSpec = getOptionalAttribute(MAX, element);
		Integer max = (maxSpec != null ? Integer.parseInt(maxSpec) : null);
		String maxLengthSpec = getOptionalAttribute(MAX_LENGTH, element);
		if (maxLengthSpec != null)
			LOGGER.warn("Attribute 'maxLength' of <stringLength> has been renamed to 'max'. " +
					"Please update your check definition");
		if (max == null && maxLengthSpec != null)
			max = Integer.parseInt(maxLengthSpec);
		
		// verify setup
		if (min == null && max == null)
			throw new SyntaxError("'min' or 'max' must be specified", XMLUtil.format(element));
		
		// create query
	    String reportColumnsSpec = getReportColumnsSpec(element);
		DefectQueryStrategy strategy = new StringLengthQueryStrategy(
				tableName, reportColumnsSpec, expression, condition, min, max);
		parent.setStrategy(strategy);
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
	    return strategy;
	}

}
