/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.ArrayUtil;
import org.databene.commons.BeanUtil;
import org.databene.commons.CollectionUtil;
import org.databene.commons.ExceptionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.databene.dbsanity.model.ValidatorStrategy;
import org.w3c.dom.Element;

/**
 * Parses &lt;validator&gt; elements.<br/><br/>
 * Created: 26.10.2011 22:27:24
 * @since 0.9.0
 * @author Volker Bergmann
 */
public class ValidatorParser extends AbstractDbSanityXMLParser {

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet("class");
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(REPORT_COLUMNS, COLUMNS, EXPRESSION);
	
	public ValidatorParser() {
		super("validator", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
		StrategyBasedCheck parent = (StrategyBasedCheck) this.parent(parentPath);
		String tableName = getTableName(parent, XMLUtil.format(element));

		// parse XML element
		String expression = getOptionalAttribute(EXPRESSION, element);
		String[] columns = parseColumns(element, false);
		if (expression != null)
			columns = new String[] { expression };
        if (ArrayUtil.isEmpty(columns))
        	throw new SyntaxError("'columns' or 'expression' attribute missing", XMLUtil.format(element));
        
		String condition = getCondition(element);
		
        String validatorClassName = element.getAttribute("class");
        if (StringUtil.isEmpty(validatorClassName))
        	throw new SyntaxError("'class' attribute is missing", XMLUtil.format(element));
        try {
        	Object validator = BeanUtil.newInstance(validatorClassName);
            for (Element childElement : XMLUtil.getChildElements(element, false, "property")) {
            	String name = getRequiredAttribute("name", childElement);
            	String value = getRequiredAttribute("value", childElement);
            	BeanUtil.setPropertyValue(validator, name, value, true, true);
            }
    		String reportColumnsSpec = getReportColumnsSpec(element);
			ValidatorStrategy strategy = new ValidatorStrategy(validator, tableName, columns, condition, reportColumnsSpec);
    		parent.setStrategy(strategy);
    		if (parent.getDefectType() == null)
    			parent.setDefectType(validator.getClass().getSimpleName());
    	    return strategy;
        } catch (RuntimeException e) {
        	if (ExceptionUtil.getRootCause(e) instanceof ClassNotFoundException)
        		throw new SyntaxError("Not a valid class name: " + validatorClassName, XMLUtil.format(element));
        	else
        		throw e;
        }
	}

}
