/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.databene.dbsanity.model.ValuesQueryStrategy;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.w3c.dom.Element;

/**
 * Checks a selector expression result to be in a given list of legal values.<br/><br/>
 * Created: 24.10.2011 15:07:08
 * @since 0.9.0
 * @author Volker Bergmann
 */
public class ValuesParser extends AbstractDbSanityXMLParser {
	
	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet(EXPRESSION);
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(REPORT_COLUMNS, "list");

	private static final String DEFAULT_DEFECT_TYPE = "values";

	public ValuesParser() {
		super("values", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
		StrategyBasedCheck parent = (StrategyBasedCheck) this.parent(parentPath);
		String tableName = getTableName(parent, XMLUtil.format(element));

		// parse column
		String expression = getRequiredAttribute(EXPRESSION, element).trim();
		String condition = getCondition(element);
		
		// parse values in 'list' attribute
		String listSpec = getOptionalAttribute("list", element);
		List values;
		if (listSpec != null)
			values = CollectionUtil.toList(StringUtil.trimAll(listSpec.split(",")));
		else
			values = new ArrayList<String>();
		// parse values in child <value> elements
		for (Element child : XMLUtil.getChildElements(element, false, "value"))
			values.add(StringUtil.trim(XMLUtil.getText(child)));
		if (values.isEmpty())
			syntaxError("<values> element must have 'list' attribute or <value> sub elements", element);
		
	    String reportColumnsSpec = getReportColumnsSpec(element);
		DefectQueryStrategy strategy = new ValuesQueryStrategy(tableName, expression, condition, values, reportColumnsSpec);
		parent.setStrategy(strategy);
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
	    return strategy;
	}
	
}
