/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;

import org.databene.commons.ArrayFormat;
import org.databene.commons.IOUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.math.Interval;
import org.databene.commons.math.Intervals;
import org.databene.commons.version.VersionNumber;
import org.databene.dbsanity.DbSanity;
import org.databene.dbsanity.html.HtmlElement;
import org.databene.dbsanity.html.HtmlEntity;
import org.databene.dbsanity.html.HtmlFileReference;
import org.databene.dbsanity.html.HtmlFont;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.html.HtmlStyle;
import org.databene.dbsanity.html.HtmlTable;
import org.databene.dbsanity.html.HtmlTableCell;
import org.databene.dbsanity.html.HtmlTableHeaderCell;
import org.databene.dbsanity.html.HtmlTableRow;
import org.databene.dbsanity.html.HtmlSimpleContentElement;
import org.databene.dbsanity.html.Renderable;
import org.databene.dbsanity.html.RenderableText;
import org.databene.dbsanity.model.DefectRow;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.html.HTMLUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Abstract implementation of the {@link ReportModule} interface.
 * It provides a {@link PrintStream} attribute named <code>out</code>
 * and methods to open and close an HTML file on this attribute.<br/><br/>
 * Created: 13.10.2010 16:39:39
 * @since 1.0
 * @author Volker Bergmann
 */
public abstract class AbstractReportModule implements ReportModule {
	
	protected final Logger logger = LoggerFactory.getLogger(getClass());
	
	protected ReportContext context;
	protected String style = DbSanity.DEFAULT_STYLE;

	public int getDashboardColSpan(ReportScope scope) {
		return 0;
	}
	
	public void setContext(ReportContext context) {
	    this.context = context;
	}
	
	public void setStyle(String style) {
    	this.style = style;
    }

	public void success(SanityCheck check) {
	}
	
	public void failure(SanityCheck check, DefectRow defectRow, String[] infoHeaders) {
	}
	
	public void error(SanityCheck check, Exception e) {
	}
	
	public void skipped(SanityCheck check) {
	}
	
	public void summary(SanityCheckSuite rootSuite) {
	}
	
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
	}
	
	public void close() {
    }
	
	// non-public helpers ----------------------------------------------------------------------------------------------

	protected HtmlPrintWriter openNewFile(File file, String title) 
			throws FileNotFoundException, UnsupportedEncodingException {
		return new HtmlPrintWriter(file, title, new HtmlStyle(new File(context.getReportRoot(), style + ".css")));
    }

	protected void closeFile(HtmlPrintWriter out) {
	    IOUtil.close(out);
    }

	protected void renderCheckInfo(SanityCheck check, HtmlPrintWriter out) {
		out.startModule("Check Info");
		out.startTable();
		printNameValueRow("header_name", check.getName(), false, false, out);
		try {
			printNameValueRow("header_source_file", check.getSourceFile().getCanonicalPath(), false, false, out);
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
		printNameValueRow("header_table", check.getTable(), false, false, out);
		printNameValueRow("header_defect_type", check.getDefectType(), false, false, out);
		printNameValueRow("header_description", check.getDescription(), true, false, out);
		printNameValueRow("header_requirement", check.getRequirement(), true, false, out);
		String version = renderVersions(check);
		printNameValueRow("header_affected_versions", version, false, false, out);
		printNameValueRow("header_tags", ArrayFormat.format(check.getTags().toArray()), true, false, out);
		printNameValueRow("header_author", check.getAuthor(), true, false, out);
		printNameValueRow("header_details", check.getDetails(), true, true, out);
		out.endTable();
		out.endModule();
	}

	protected String renderVersions(SanityCheck check) {
		Intervals<VersionNumber> versions = check.getAffectedVersions();
		StringBuilder builder = new StringBuilder();
		for (int i = 0; i < versions.intervalCount(); i++)
			builder.append(i > 0 ? ", " : "").append(renderInterval(versions.getInterval(i)));
		String result = builder.toString();
		if ("any".equals(result))
			result = context.localize("any_version");
		return result;
	}
	
	static String renderInterval(Interval<VersionNumber> interval) {
		VersionNumber min = interval.getMin();
		VersionNumber max = interval.getMax();
		if (min != null) {
			// min != null
			if (max != null)
				return interval.toString();
			else if (interval.isMinInclusive())
				return "since " + min;
			else
				return "after " + min;
		} else {
			// min == null
			if (max == null)
				return "any";
			else if (interval.isMaxInclusive())
				return "until " + max + " inclusively";
			else
				return "before " + max;
		}
	}

	protected void printNameValueRow(String name, String value, boolean dropIfEmpty, boolean pre, HtmlPrintWriter out) {
		if (dropIfEmpty && StringUtil.isEmpty(value))
			return;
		out.startTableRow();
		out.render(headerCell(context.localize(name) + ":").leftAligned().topAligned());
		out.startTableCell("align", "left");
		if (!StringUtil.isEmpty(value)) {
			if (pre)
				out.startPreformatted();
			out.print(HTMLUtil.escape(value));
			if (pre)
				out.endPreformatted();
		}
		out.endTableCell();
		out.endTableRow();
	}
	
	protected String localize(String key) {
		return context.localize(key);
	}
	
	
	// histogram rendering ---------------------------------------------------------------------------------------------
	
	protected <T> void renderHistogram(HistogramModel<T> criterion,
			List<T> elements, int limit, File detailsFile, HtmlPrintWriter out) {
		out.startTable("width", "100%", "cellpadding", "2px");
		if (elements.size() == 0) {
			out.renderRow(headerCell(nonBreakableSpace()));
		} else {
			boolean halfReached = false;
			long defectsSoFar = 0;
			long maxCount = maxValue(elements, criterion);
			long totalCount = sumOfValues(elements, criterion);
			int labelWidth = criterion.getLabelWidth();
			int numberOfColumns = 2;
			out.startTableRow();
			int remainingWidth = 100;
			if (criterion.isRenderVerdict()) {
				out.render(headerCell(localize("verdict")).withAlignment("center").withWidth("5%"));
				remainingWidth -= 5;
				numberOfColumns = 3;
			}
			out.render(headerCell(criterion.getLabelColumnTitle()).leftAligned().withWidth(labelWidth + "%"));
			out.render(headerCell(criterion.getBarColumnTitle()).leftAligned().withWidth((remainingWidth - labelWidth) + "%"));
			out.endTableRow();
			for (int i = 0; (i < limit || limit < 0) && i < elements.size(); i++) {
				T element = elements.get(i);
				Renderable label = criterion.getLabel(element);
				if (label == null)
					label = new RenderableText("unknown");
				long value = criterion.getValue(element);
				String percentage = String.valueOf((long) (value * 100 / (maxCount != 0 ? maxCount : 1))) + "%";
				out.startTableRow();
				if (criterion.isRenderVerdict()) {
					SanityCheckVerdict verdict = criterion.getVerdict(element);
					out.render(cell(context.verdictImage(verdict)).withAlignment("center"));
				}
				out.render(cell(label).leftAligned());
				out.startTableCell("align", "left");
				numberBar(value, percentage, criterion.getColor(), context, out);
				out.endTableCell();
				out.endTableRow();
				defectsSoFar += value;
				if (criterion.isMedianRuler() 
						&& !halfReached // marker not yet painted 
						&& defectsSoFar > totalCount / 2 // 50% or more 
						&& defectsSoFar < totalCount) { // if it is the only check, we don't need the marker
					printHalfVolumeRuler(defectsSoFar * 100 / totalCount, numberOfColumns, out);
					halfReached = true;
				}
			}
			if (limit >= 0 && limit < elements.size()) {
				// if the number of defective elements exceeds the limit, print a [more...] reference
				out.startTableRow();
				if (criterion.isRenderVerdict())
					out.render(cell("...").centered());
				out.render(cell().withContent(new HtmlFileReference(localize("more"), detailsFile)));
				out.render(cell("..."));
				out.endTableRow();
			}
		}
		out.endTable();
	}

	private <T> long sumOfValues(List<T> elements, HistogramModel<T> criterion) {
		long sum = 0;
		for (T element : elements)
			sum += criterion.getValue(element);
		return sum;
	}

	private <T> long maxValue(List<T> elements, HistogramModel<T> criterion) {
		Long max = null;
		for (T element : elements) {
			Long value = criterion.getValue(element);
			if (max == null || value > max)
				max = value;
		}
		return max;
	}

	private void printHalfVolumeRuler(long percentage, int colspan, HtmlPrintWriter out) {
		out.renderRow(
			cell().withColspan(colspan).withContent(table().withWidth("100%").withRows(row(
				cell().withWidth("0%").withContent(font().withSize("-1").withColor("#aaa").withContent(new RenderableText(percentage + "%"))),
				cell(horizontalRule()).withWidth("100%"))
			)
		));
	}

	public static void numberBar(long n, String width, String color, ReportContext context, HtmlPrintWriter out) {
		bar(n, new RenderableText(context.formatLong(n)), width, color, context, out);
	}

	public static void durationBar(long n, String width, String color, ReportContext context, HtmlPrintWriter out) {
		bar(n, context.formatDuration(n), width, color, context, out);
	}

	private static void bar(long n, Renderable label, String width, String color, ReportContext context, HtmlPrintWriter out) {
		out.startTable("width", width, "cellpadding", "0", "cellspacing", "0");
		out.startTableRow();
		if (n > 0) {
			out.startTableCell("width", "100%", "align", "left");
			out.printInlinePanel(label, "style", "background: " + color +"; color: #ffffff; font-size: small");
			out.endTableCell();
		}
		out.endTableRow();
		out.endTable();
	}

	// generic HTML rendering helpers ----------------------------------------------------------------------------------
	
	protected HtmlTable table(HtmlTableRow... rows) {
		return new HtmlTable(rows);
	}
	
	protected HtmlTableRow row(HtmlTableCell... cells) {
		return new HtmlTableRow(cells);
	}
	
	protected HtmlTableCell cell() {
		return new HtmlTableCell("");
	}
	
	protected HtmlTableCell cell(String content) {
		return new HtmlTableCell(content);
	}
	
	protected HtmlTableCell cell(Renderable content) {
		return new HtmlTableCell(content);
	}
	
	protected HtmlTableCell cell(int content) {
		return new HtmlTableCell(content);
	}
	
	protected HtmlTableHeaderCell headerCell(String content) {
		return new HtmlTableHeaderCell(content);
	}

	protected HtmlTableHeaderCell headerCell(Renderable content) {
		return new HtmlTableHeaderCell(content);
	}

	protected Renderable bold(String content) {
		return new HtmlSimpleContentElement("b", content);
	}

	protected Renderable bold(Renderable content) {
		return new HtmlSimpleContentElement("b", content);
	}
	
	protected Renderable preformatted(Renderable renderable) {
		return new HtmlSimpleContentElement("pre", renderable);
	}

	protected Renderable nonBreakableSpace() {
		return new HtmlEntity("nbsp");
	}

	protected HtmlFont font() {
		return new HtmlFont();
	}

	protected Renderable horizontalRule() {
		return new HtmlElement<HtmlElement<?>>("hr", true);
	}

}
