/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import static org.databene.dbsanity.model.SanityCheckVerdict.ERROR;
import static org.databene.dbsanity.model.SanityCheckVerdict.FAILED;
import static org.databene.dbsanity.model.SanityCheckVerdict.NOT_APPLICABLE;

import java.io.File;
import java.io.IOException;

import org.databene.dbsanity.html.HtmlFileReference;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.dbsanity.util.CheckVisitor;

/**
 * Abstract parent class for {@link ReportModule}s that create a report summary.<br/><br/>
 * Created: 23.01.2012 09:20:27
 * @since 0.9.3
 * @author Volker Bergmann
 */
public abstract class AbstractSummaryReportModule<E> extends AbstractReportModule {
	
	private final String i18nPrefix;
	private final String summaryFileName;

	public AbstractSummaryReportModule(String i18nPrefix, String summaryFileName) {
		this.i18nPrefix = i18nPrefix;
		this.summaryFileName = summaryFileName;
	}
	
	public String getLinkLabel() {
		return localize(i18nPrefix + "_download");
	}
	
	// ReportModule interface implementation ---------------------------------------------------------------------------
	
	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 1 : 0);
	}
	
	@Override
	public void summary(SanityCheckSuite rootSuite) {
		createSummaryFile(rootSuite);
	}

	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
		out.startModule(localize(i18nPrefix + "_report"));
		out.render(new HtmlFileReference(getLinkLabel(), getFile(suite.getRootSuite())));
		out.endModule();
	}
	
	// private helpers -------------------------------------------------------------------------------------------------
	
	protected File getFile(SanityCheckSuite rootSuite) {
		return new File(rootSuite.getReportFolder(), summaryFileName);
	}

	private void createSummaryFile(SanityCheckSuite rootSuite) {
		E out = null;
		try {
			out = startSummaryFile(rootSuite);
			rootSuite.accept(new CheckExporter(out));
		} catch (IOException e) {
			logger.error("Error creating report in " + getClass(), e);
		} finally {
			if (out != null)
				endSummaryFile(out);
		}
	}

	protected abstract E startSummaryFile(SanityCheckSuite rootSuite) throws IOException;
	protected abstract void exportCheck(SanityCheck check, E out);
	protected abstract void endSummaryFile(E out);

	
	public class CheckExporter extends CheckVisitor {
		
		private E out;
		
		public CheckExporter(E out) {
			this.out = out;
		}

		public void visitCheck(SanityCheck check) {
			SanityCheckVerdict verdict = check.getVerdict();
			if (verdict != NOT_APPLICABLE && (!context.isCondensed() || verdict == ERROR || verdict == FAILED))
				exportCheck(check, out);
		}

	}

}
