/*
 * (c) Copyright 2010-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;

import org.databene.commons.OrderedSet;
import org.databene.commons.StringUtil;
import org.databene.dbsanity.html.HtmlFileReference;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.dbsanity.util.CheckVisitor;
import org.databene.jdbacl.model.DBTable;
import org.databene.jdbacl.model.Database;

/**
 * Checks the sanity checks for completeness and correctness.<br/><br/>
 * Created: 08.11.2010 12:26:39
 * @since 1.0
 * @author Volker Bergmann
 */
public class CheckAssessmentModule extends AbstractReportModule {

	private Database database;
	private String excludedTables;
	private Set<SanityCheck> checksWithoutTable;
	private Set<SanityCheck> checksWithoutDefectType;
	private Set<SanityCheck> checksWithError;
	private List<String> uncheckedTables;

	public CheckAssessmentModule(Database database, String excludedTables) {
		this.database = database;
		this.excludedTables = excludedTables;
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 1 : 0);
	}

	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
		out.startModule(localize("check_quality_assessment"));
		out.startTable("width", "100%", "cellpadding", "3");
		renderSummaryLine(checksWithError, "faulty_checks", out);
		renderSummaryLine(checksWithoutTable, "checks_without_table", out);
		renderSummaryLine(checksWithoutDefectType, "checks_without_defect_type", out);
		if (checksWithoutTable.size() == 0 && checksWithoutDefectType.size() == 0 && checksWithError.size() == 0)
			out.render(row(cell("all_checks_ok")));
		renderSummaryLine(uncheckedTables, "unchecked_tables", out);
		out.endTable();
		out.endModule(localize("show_detail_page"), getDetailsFile(out));
	}
	
	private void renderSummaryLine(Collection<?> elements, String key, HtmlPrintWriter out) {
		if (!elements.isEmpty())
			out.renderRow(
				cell(elements.size()).withStyle("color:red").withAlignment("right"),
				cell(localize(key)).withStyle("color:red").withAlignment("left")
			);
	}

	@Override
	public void summary(SanityCheckSuite rootSuite) {
		scanSuite(rootSuite);
		createDetailsPage(rootSuite);
	}

	private void scanSuite(SanityCheckSuite suite) {
		this.checksWithoutTable = new OrderedSet<SanityCheck>();
		this.checksWithoutDefectType = new OrderedSet<SanityCheck>();
		this.checksWithError = new OrderedSet<SanityCheck>();
		collectTables();
		suite.accept(new CheckAssessmentVisitor());
	}

	private void collectTables() {
		List<DBTable> tables = database.getTables();
		this.uncheckedTables = new ArrayList<String>(tables.size());
		Pattern pattern = (excludedTables != null ? Pattern.compile(excludedTables) : null);
		for (DBTable table : tables) {
			String tableName = table.getName().toUpperCase();
			if (pattern == null || !pattern.matcher(tableName).matches())
				uncheckedTables.add(tableName);
		}
		Collections.sort(this.uncheckedTables);
	}

	private void createDetailsPage(SanityCheckSuite rootSuite) {
		String title = "Sanity Check Assessment";
		HtmlPrintWriter out = null;
		File file = new File(rootSuite.getReportFolder(), "check_assessment.html");
		try {
			out = openNewFile(file, title);
			context.printNavBar(file, out);
			faultyCheckDiv(out, file);
			missingTableDiv(out, file);
			missingDefectTypeDiv(out);
			uncheckedTablesDiv(-1, 4, out);
		} catch (Exception e) {
			throw new RuntimeException("Error writing quality assessment file: " + file, e);
		} finally {
			super.closeFile(out);
		}
	}

	private void faultyCheckDiv(HtmlPrintWriter out, File file) {
		try {
			out.startModule(localize("faulty_checks"));
			if (checksWithError.size() > 0) {
				out.startTable("cellpadding", "3");
				out.renderRow(
					headerCell(context.formatLong(checksWithError.size())).withClass("defect").withAlignment("right"),
					headerCell(localize("faulty_checks")).withClass("defect").withAlignment("left")
				);
				for (SanityCheck check : checksWithError) {
					HtmlFileReference reference = ReportUtil.docOrDefectFileRef(check);
					out.render(row(cell(reference).withColspan(2)));
				}
				out.endTable();
			} else {
				out.println(localize("none"));
			}
			out.endModule();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	private void missingTableDiv(HtmlPrintWriter out, File file) {
		try {
			out.startModule(localize("checks_without_table"));
			if (checksWithoutTable.size() > 0) {
				out.startTable("cellpadding", "3");
				out.renderRow(
					headerCell(context.formatLong(checksWithoutTable.size())).withClass("defect").withAlignment("right"),
					headerCell(localize("checks_without_table")).withClass("defect").withAlignment("left")
				);
				for (SanityCheck check : checksWithoutTable)
					out.render(row(cell().withColspan(2).withContent(ReportUtil.docRef(check, context))));
				out.endTable();
			} else {
				out.println(localize("none"));
			}
			out.endModule();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	private void missingDefectTypeDiv(HtmlPrintWriter out) {
		out.startModule(localize("checks_without_defect_type"));
		if (checksWithoutDefectType.size() > 0) {
			out.startTable("cellpadding", "3");
			out.renderRow(
				headerCell(context.formatLong(checksWithoutDefectType.size())).withClass("defect").withAlignment("right"),
				headerCell(localize("checks_without_defect_type")).withClass("defect").withAlignment("left")
			);
			for (SanityCheck check : checksWithoutDefectType)
				out.render(row(cell(ReportUtil.docRef(check, context)).withColspan(2)));
			out.endTable();
		} else {
			out.println(localize("none"));
		}
		out.endModule();
	}

	private void uncheckedTablesDiv(int countLimit, int columnCount, HtmlPrintWriter out) {
		// order defect type counts in descending order
		String title;
		if (uncheckedTables.size() == 0)
			title = localize("no_unchecked_tables");
		else
			title = context.formatLong(uncheckedTables.size()) + " " + localize("unchecked_tables");
		out.startModule(title);
		if (uncheckedTables.size() > 0) {
			out.startTable("width", "100%");
			out.renderRow(headerCell(localize("tables")).withColspan(2).withAlignment("left"));
			for (int i = 0; (i < countLimit || countLimit < 0) && i < uncheckedTables.size(); i += columnCount) {
				out.startTableRow();
				for (int j = i; j < i + columnCount; j++)
					if ((j < countLimit || countLimit < 0) && j < uncheckedTables.size())
						out.render(cell(uncheckedTables.get(j)).withAlignment("left"));
				out.endTableRow();
			}
			if (countLimit >= 0 && countLimit < uncheckedTables.size()) {
				out.renderRow(
					cell(new HtmlFileReference(localize(ReportContext.MORE), getDetailsFile(out))).withAlignment("left"),
					cell("")
				);
			}
			out.endTable();
		} else {
			out.println(localize("none"));
		}
		out.endModule();
	}

	private File getDetailsFile(HtmlPrintWriter out) {
		return new File(out.getFile().getParentFile(), "check_assessment.html");
	}

	class CheckAssessmentVisitor extends CheckVisitor {
		protected void visitCheck(SanityCheck check) {
			if (StringUtil.isEmpty(check.getTable()))
				checksWithoutTable.add(check);
			if (StringUtil.isEmpty(check.getDefectType()))
				checksWithoutDefectType.add(check);
			if (check.getVerdict() == SanityCheckVerdict.ERROR)
				checksWithError.add(check);
			String table = check.getTable();
			if (table != null) {
				table = table.toUpperCase();
				if (uncheckedTables.contains(table))
					uncheckedTables.remove(table);
			}
		}
	}
	
}
