/*
 * (c) Copyright 2010-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.databene.commons.Visitor;
import org.databene.commons.comparator.LongComparator;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.html.Renderable;
import org.databene.dbsanity.html.RenderableText;
import org.databene.dbsanity.model.Reportable;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.util.CheckVisitor;

/**
 * Lists all checks with defects ordered by descending defect count.<br/><br/>
 * Created: 15.12.2010 09:48:25
 * @since 0.6
 * @author Volker Bergmann
 */
public class DefectsPerCheckModule extends AbstractReportModule {

	private static final String DETAILS_FILE_NAME = "defects_per_check.html";
	private List<SanityCheck> defectiveChecks;
	private long totalDefects;

	public DefectsPerCheckModule() {
		this.defectiveChecks = new ArrayList<SanityCheck>();
		this.totalDefects = 0;
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 2 : 0);
	}
	
	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
		div(5, out);
	}
	
	@Override
	public void summary(SanityCheckSuite rootSuite) {
		scanSuite(rootSuite);
		createDetailsPage(rootSuite);
	}

	private void scanSuite(SanityCheckSuite suite) {
		// fetch all defective checks
		Visitor<Reportable> visitor = new CheckVisitor() {
			public void visitCheck(SanityCheck check) {
				if (check.getDefectCount() > 0) {
					defectiveChecks.add(check);
					totalDefects += check.getDefectCount();
				}
			}
		};
		suite.accept(visitor);
		
		// order checks by descending defect count
		Collections.sort(defectiveChecks, new Comparator<SanityCheck>() {
			public int compare(SanityCheck count1, SanityCheck count2) {
				return - LongComparator.compare(count1.getDefectCount(), count2.getDefectCount());
			}
		});
	}

	private void createDetailsPage(SanityCheckSuite rootSuite) {
		if (defectiveChecks.size() == 0)
			return;
		String title = localize("defects_per_check");
		HtmlPrintWriter out = null;
		File file = new File(rootSuite.getReportFolder(), DETAILS_FILE_NAME);
		try {
			out = openNewFile(file, title);
			context.printNavBar(file, out);
			div(-1, out);
		} catch (Exception e) {
			throw new RuntimeException("Error writing defects per check file: " + file, e);
		} finally {
			super.closeFile(out);
		}
	}

	private void div(int limit, HtmlPrintWriter out) {
		String title;
		if (limit >= 0)
			title = localize("defects_per_check");
		else
			title = MessageFormat.format("n_checks_with_defects", context.formatLong(defectiveChecks.size()));
		out.startModule(title);
		renderHistogram(new DefectsPerCheckHistogramModel(context), defectiveChecks, limit, getDetailsFile(out), out);
		out.endModule();
	}

	private File getDetailsFile(HtmlPrintWriter out) {
		return new File(out.getFile().getParentFile(), DETAILS_FILE_NAME);
	}



	public class DefectsPerCheckHistogramModel extends HistogramModel<SanityCheck> {

		public DefectsPerCheckHistogramModel(ReportContext context) {
			super("check", "defect_count", false, "#" + ReportContext.DEFECT_COLOR, 30, true, context);
		}
		
		@Override
		public Renderable getLabel(SanityCheck element) {
			return new RenderableText(element.getName());
		}

		@Override
		public long getValue(SanityCheck element) {
			return element.getDefectCount();
		}

		@Override
		public Renderable formatValue(long value) {
			return new RenderableText(context.formatLong(value));
		}
	}

}
