/*
 * (c) Copyright 2010-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.util.CheckCriteriaAggregator;

/**
 * Reports the defects per table.<br/><br/>
 * Created: 08.12.2010 20:46:25
 * @since 1.0
 * @author Volker Bergmann
 */
public class DefectsPerTableModule extends AbstractReportModule {
	
	private static final String DETAILS_FILE_NAME = "defects_per_table.html";
	
	private List<Entry<String, Long>> counts;

	public DefectsPerTableModule() {
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 1 : 0);
	}
	
	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
		div(5, out);
	}
	
	@Override
	public void summary(SanityCheckSuite rootSuite) {
		scanSuite(rootSuite);
		createDetailsPage(rootSuite);
	}

	private void scanSuite(SanityCheckSuite suite) {
		CheckCriteriaAggregator visitor = new CheckCriteriaAggregator() {
			protected long weight(SanityCheck check) {
				return check.getDefectCount();
			}
			protected Set<String> criteria(SanityCheck check) {
				String table = StringUtil.emptyToNull(StringUtil.trim(check.getTable()));
				table = (table != null ? table.toUpperCase() : localize("undefined"));
				return CollectionUtil.toSet(table);
			}
		};
		suite.accept(visitor);
		Map<String, Long> weights = visitor.getWeights();
		// order defect type counts in descending order
		counts = new ArrayList<Map.Entry<String, Long>>(weights.entrySet());
		Collections.sort(counts, new Comparator<Map.Entry<String, Long>>() {
			public int compare(Entry<String, Long> count1, Entry<String, Long> count2) {
				return - count1.getValue().compareTo(count2.getValue());
			}
		});
	}

	private void createDetailsPage(SanityCheckSuite rootSuite) {
		if (counts.size() == 0)
			return;
		String title = localize("defects_per_table");
		HtmlPrintWriter out = null;
		File file = new File(rootSuite.getReportFolder(), DETAILS_FILE_NAME);
		try {
			out = openNewFile(file, title);
			context.printNavBar(file, out);
			div(-1, out);
		} catch (Exception e) {
			throw new RuntimeException("Error writing defects per table file: " + file, e);
		} finally {
			super.closeFile(out);
		}
	}

	private void div(int limit, HtmlPrintWriter out) {
		String title;
		if (limit >= 0)
			title = localize("defects_per_table");
		else
			title = MessageFormat.format(localize("n_defects_per_table"), context.formatLong(counts.size()));
		out.startModule(title);
		renderHistogram(new MapEntryHistogramModel("table", context), counts, limit, getDetailsFile(out), out);
		out.endModule();
	}

	private File getDetailsFile(HtmlPrintWriter out) {
		return new File(out.getFile().getParentFile(), DETAILS_FILE_NAME);
	}

}
