/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.databene.commons.IOUtil;
import org.databene.commons.collection.OrderedNameMap;
import org.databene.dbsanity.html.HtmlFileReference;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.html.HtmlSpan;
import org.databene.dbsanity.html.Renderable;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.util.CheckTagAggregator;
import org.databene.dbsanity.util.TagCloudUtil;

/**
 * Lists the number of defects per tag.<br/><br/>
 * Created: 11.11.2011 17:18:19
 * @since 0.9.2
 * @author Volker Bergmann
 */
public class DefectsPerTagModule extends AbstractReportModule {
	
	private static final String DETAILS_FILE_NAME = "defects_per_tag.html";

	private static final int DASHBOARD_ENTRIES = 10;

	private static final int MIN_FONT_SIZE = 80;
	private static final int MAX_FONT_SIZE = 200;

	private Map<String, Long> weights;
	
	public DefectsPerTagModule() {
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 1 : 0);
	}
	
	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
		div(DASHBOARD_ENTRIES, out);
	}
	
	@Override
	public void summary(SanityCheckSuite rootSuite) {
		scanSuite(rootSuite);
		createDetailsPage(rootSuite);
	}

	private void scanSuite(SanityCheckSuite suite) {
		CheckTagAggregator visitor = new CheckTagAggregator(
				context.isUsingNamespaces(), context.isAuthorInCloud(), context.isTableInCloud()) {
			@Override
			protected long weight(SanityCheck check) {
				return check.getDefectCount();
			}
		};
		suite.accept(visitor);
		weights = visitor.getWeights();
	}

	private void createDetailsPage(SanityCheckSuite rootSuite) {
		if (weights.size() == 0)
			return;
		String title = localize("defects_per_tag");
		HtmlPrintWriter out = null;
		File file = new File(rootSuite.getReportFolder(), DETAILS_FILE_NAME);
		try {
			out = openNewFile(file, title);
			context.printNavBar(file, out);
			div(-1, out);
		} catch (Exception e) {
			throw new RuntimeException("Error writing defect type count file: " + file, e);
		} finally {
			IOUtil.close(out);
		}
	}

	private void div(int limit, HtmlPrintWriter out) {
		// order defect type counts in descending order
		List<Entry<String, Long>> counts = new ArrayList<Map.Entry<String, Long>>(weights.entrySet());
		Collections.sort(counts, new Comparator<Map.Entry<String, Long>>() {
			public int compare(Entry<String, Long> count1, Entry<String, Long> count2) {
				return - count1.getValue().compareTo(count2.getValue());
			}
		});
		String title;
		if (limit >= 0)
			title = localize("defects_per_tag");
		else
			title = MessageFormat.format(localize("n_tags_with_defects"), context.formatLong(weights.size()));
		out.startModule(title);
		if (counts.size() > 0) {
			renderTagCloud(counts, limit, out);
			if (limit <= 0) {
				out.printLineBreak();
				out.printLineBreak();
				renderBars(limit, out, counts);
			}
		} else {
			out.println(localize("none"));
		}
		out.endModule();
	}

	private void renderTagCloud(List<Entry<String, Long>> counts, int limit, HtmlPrintWriter out) {
		int n = counts.size();
		if (limit > 0)
			n = Math.min(n, limit); 
		Map<String, Long> usedCounts = OrderedNameMap.<Long>createCaseInsensitiveMap();
		for (int i = 0; i < n; i++) {
			Entry<String, Long> entry = counts.get(i);
			usedCounts.put(entry.getKey(), entry.getValue());
		}
		TagCloudUtil.printTagCloud(usedCounts, "#" + ReportContext.DEFECT_COLOR, MIN_FONT_SIZE, MAX_FONT_SIZE, out);
		if (limit > 0) {
			out.print(" ");
			Renderable more = new HtmlSpan().withStyle("font-size:" + MIN_FONT_SIZE + "%").withContent(localize(ReportContext.MORE));
			out.render(new HtmlFileReference(more, getDetailsFile(out)));
		}
	}

	protected void renderBars(int limit, HtmlPrintWriter out, List<Entry<String, Long>> counts) {
		renderHistogram(new MapEntryHistogramModel("tag", context), counts, limit, getDetailsFile(out), out);
	}

	private File getDetailsFile(HtmlPrintWriter out) {
		return new File(out.getFile().getParentFile(), DETAILS_FILE_NAME);
	}

}
