/*
 * (c) Copyright 2010-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.databene.commons.comparator.IntComparator;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.html.Renderable;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;

/**
 * New module that lists the slow checks.<br/><br/>
 * Created: 11.11.2010 20:08:32
 * @since 1.0
 * @author Volker Bergmann
 */
public class ExecutionTimeModule extends AbstractReportModule {
	
	private static final String DETAILS_FILE_NAME = "execution_times.html";
	List<SanityCheck> checks;

	public ExecutionTimeModule() {
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 2 : 0);
	}
	
	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
		div(5, out);
	}
	
	@Override
	public void summary(SanityCheckSuite rootSuite) {
		checks = checkSuite(rootSuite);
		createDetailsPage(checks, rootSuite);
	}

	private List<SanityCheck> checkSuite(SanityCheckSuite suite) {
		List<SanityCheck> checks = ReportUtil.aggregateChecks(suite, true);
		Collections.sort(checks, new Comparator<SanityCheck>() {
			public int compare(SanityCheck c1, SanityCheck c2) {
				return IntComparator.compare(c2.getExecutionTime(), c1.getExecutionTime());
			}
		});
		return checks;
	}

	private void createDetailsPage(List<SanityCheck> checks, SanityCheckSuite rootSuite) {
		String title = localize("execution_times");
		HtmlPrintWriter out = null;
		File file = new File(rootSuite.getReportFolder(), DETAILS_FILE_NAME);
		try {
			out = openNewFile(file, title);
			context.printNavBar(rootSuite.getReportFolder(), out);
			div(-1, out);
		} catch (Exception e) {
			throw new RuntimeException("Error writing execution times file: " + file, e);
		} finally {
			super.closeFile(out);
		}
	}

	private void div(int limit, HtmlPrintWriter out) {
		// order defect type counts in descending order
		if (!checks.isEmpty()) {
			out.startModule(localize("execution_times"));
			ExecutionTimeHistogramModel criterion = new ExecutionTimeHistogramModel(context);
			renderHistogram(criterion, checks, limit, getDetailsFile(out), out);
			out.endModule();
		}	
	}

	private File getDetailsFile(HtmlPrintWriter out) {
		return new File(out.getFile().getParentFile(), DETAILS_FILE_NAME);
	}

	
	
	class ExecutionTimeHistogramModel extends HistogramModel<SanityCheck> {
		
		public ExecutionTimeHistogramModel(ReportContext context) {
			super("check", "execution_time", true, "#006688", 30, true, context);
		}

		@Override
		public Renderable getLabel(SanityCheck element) {
			return ReportUtil.docOrDefectFileRef(element);
		}

		@Override
		public long getValue(SanityCheck element) {
			return element.getExecutionTime();
		}

		@Override
		public SanityCheckVerdict getVerdict(SanityCheck element) {
			return element.getVerdict();
		}

		@Override
		public Renderable formatValue(long value) {
			return context.formatDuration(value);
		}
	}
	
}
