/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;

import org.databene.commons.FileUtil;
import org.databene.commons.IOUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.parser.IdentityCheckParser;
import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * Exports a check file in HTML format.<br/><br/>
 * Created: 12.11.2010 18:11:58
 * @since 0.4
 * @author Volker Bergmann
 */
public class HTMLCheckExporter {
	
	TransformerHandler handler;
	int indent = 0;
	boolean sql = false;
	boolean pre = false;
	private Element root;
	private ReportContext context;

	public void export(SanityCheckFile file, ReportContext context, String style) throws IOException {
		Document document = XMLUtil.parse(file.getSource().getAbsolutePath());
		this.root = document.getDocumentElement();
		export(file.getSource(), root, file.getDocPage(), context, style);
	}

	private void export(File sourceFile, Element root, File targetFile, ReportContext context, String style) {
		this.context = context;
		FileUtil.ensureDirectoryExists(targetFile.getParentFile());
		try {
        	OutputStream out = new FileOutputStream(targetFile);
			handler = createHandler(out);
			transformFile(root, sourceFile, targetFile, context.getReportRoot(), style);
			handler.endDocument();
			IOUtil.close(out);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	private static TransformerHandler createHandler(OutputStream out)
			throws TransformerFactoryConfigurationError,
			TransformerConfigurationException {
		SAXTransformerFactory tf = (SAXTransformerFactory) SAXTransformerFactory.newInstance();
		TransformerHandler handler = tf.newTransformerHandler();
		Transformer serializer = handler.getTransformer();
		serializer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
		serializer.setOutputProperty(OutputKeys.INDENT, "yes");
		handler.setResult(new StreamResult(out));
		return handler;
	}

	private void transformFile(Element root, File sourceFile, File targetFile, File reportRoot, String style) throws SAXException {
		String title = "Source View of " + sourceFile.getName();
		printStartElement("html");
		printHead(title, reportRoot, style, targetFile);
		printStartElement("body");
		printTitle(title);
		printNavBar(targetFile, reportRoot);
		printSourceFileContent(root, sourceFile.getName());
		printEndElement("body");
		printEndElement("html");
	}

	private void printSourceFileContent(Element root, String title) throws SAXException {
		printStartElement("div", "class", "module");
		printStartElement("div", "class", "mod_header");
		printStartElement("h3");
		printText(title);
		printEndElement("h3");
		printEndElement("div");
		printStartElement("div", "class", "mod_body");
		printSourceElementRecursively(root);
		printEndElement("div");
		printEndElement("div");
	}

	public void printHead(String title, File reportRoot, String style,
			File targetFile) throws SAXException {
		printStartElement("head");
		printStartElement("title");
		printText(title);
		printEndElement("title");
		File styleFile = new File(reportRoot, style + ".css");
		printStartElement("link", "href", FileUtil.relativePath(targetFile, styleFile, '/'), "media", "all", "rel", "stylesheet", "type", "text/css"); 
		printEndElement("head");
	}

	public void printTitle(String title) throws SAXException {
		printStartElement("div", "class", "page_header");
		printStartElement("h1");
		printText(title);
		printEndElement("h1");
		printEndElement("div");
	}

	private void printNavBar(File targetFile, File reportRoot) throws SAXException {
		printStartElement("div", "class", "nav_bar");
		printStartElement("table", "width", "100%");
		printStartElement("tr");
		
		// dashboard link
		printStartElement("td", "align", "left");
		String href = FileUtil.relativePath(targetFile.getParentFile(), reportRoot, '/') + "/index.html";
		printStartElement("a", "href", href);
		printText(context.localize("label_home"));
		printEndElement("a");
		printEndElement("td");
		
		// Project homepage link
		printStartElement("td", "align", "right");
		printStartElement("a", "href", "http://databene.org/dbsanity", "target", "_blank");
		printText(context.localize(ReportContext.HELP_KEY));
		printEndElement("a");
		printEndElement("td");
		
		printEndElement("tr");
		printEndElement("table");
		printEndElement("div");
	}

	private void printSourceElementRecursively(Element e) throws SAXException {
		// open element
		printStartDiv();
		String name = e.getNodeName();
		if ("test".equals(name) || "check".equals(name) || "identity".equals(name)) {
			String testName = (
					"identity".equals(name) ?
					IdentityCheckParser.createCheckName(e.getAttribute("table")) :
					e.getAttribute("name"));
			printEmptyElement("hr");
			printStartElement("a", "name", ReportUtil.normalizeUrl(testName));
			printStartElement("h2", "style", "color:#048;");
			printText(testName);
			printEndElement("h2");
			printEndElement("a");
			sql = ("identity".equals(name));
		} else if ("sql".equals(name)) {
			sql = true;
		}
		printText("<" + name);
		Map<String, String> attributes = XMLUtil.getAttributes(e);
		for (Entry<String, String> attribute : attributes.entrySet())
			printText(" " + attribute.getKey() + "=\"" + attribute.getValue() + '"');
		if (e.hasChildNodes()) {
			printText(">");
			indent++;
			
			transformSubElements(e);
			
			// close element
			if (pre) {
				printEndElement("pre");
				sql = pre = false;
			}
			indent--;
			if (e == root)
				printEmptyElement("hr");
			printText("</" + name + ">");
		} else
			printText("/>");

		printEndElement("div");
	}

	private void printEmptyElement(String name) throws SAXException {
		printStartElement(name);
		printEndElement(name);
	}

	private void transformSubElements(Element e) throws SAXException {
		NodeList childNodes = e.getChildNodes();
		for (int i = 0; i < childNodes.getLength(); i++) {
			Node node = childNodes.item(i);
			if (node instanceof Element)
				printSourceElementRecursively((Element) node);
			else if (node instanceof Text)
				transformTextNode((Text) node);
			else if (node instanceof Comment)
				transformCommentNode((Comment) node);
			else
				throw new UnsupportedOperationException("node type: " + node.getClass());
		}
	}

	private void transformCommentNode(Comment node) throws SAXException {
		printText("<!--" + node.getTextContent() + "-->");
	}

	private void transformTextNode(Text node) throws SAXException {
		String text = node.getWholeText();
		if (sql && !pre) {
			if (!StringUtil.isEmpty(text.trim())) {
				printStartElement("pre", "style", "font-family:sans-serif;font-weight:bold;");
				pre = true;
			}
		}
		if (pre)
			text = StringUtil.removeEmptyLines(text);
		printText(text);
	}

	private void printText(String text) throws SAXException {
		handler.characters(text.toCharArray(), 0, text.length());
	}

	private void printStartElement(String name, String... attributes) throws SAXException {
		AttributesImpl htmlAttributes = new AttributesImpl();
		for (int i = 0; i < attributes.length; i += 2)
			htmlAttributes.addAttribute("", "", attributes[i], "", attributes[i+1]);
		handler.startElement("", "", name, htmlAttributes);
	}

	private void printStartDiv(String... attributes) throws SAXException {
		AttributesImpl htmlAttributes = new AttributesImpl();
		for (int i = 0; i < attributes.length; i += 2)
			htmlAttributes.addAttribute("", "", attributes[i], "", attributes[i+1]);
		htmlAttributes.addAttribute("", "", "style", "", "text-indent:" + indent + "cm;");
		handler.startElement("", "", "div", htmlAttributes);
	}

	private void printEndElement(String name) throws SAXException {
		handler.endElement("", "", name);
	}

}
