/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.util.Map;

import org.databene.commons.ArrayFormat;
import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.TimeUtil;
import org.databene.commons.version.VersionInfo;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.util.CheckTagAggregator;
import org.databene.jdbacl.JDBCConnectData;

/**
 * {@link ReportModule} that provides general information about database and 
 * report execution.<br/><br/>
 * Created: 08.11.2010 17:10:54
 * @since 1.0
 * @author Volker Bergmann
 */
public class InfoModule extends AbstractReportModule {

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return 1;
	}
	
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
        JDBCConnectData connectData = (JDBCConnectData) context.get("connectData");
        out.startModule(localize("settings"));
	    renderInfo(connectData, out);
        renderStats(suite, out);
        renderCheckedTags(suite, out);
		out.endModule();
    }

	protected void renderInfo(JDBCConnectData connectData, HtmlPrintWriter out) {
        out.startTable("cellpadding", "3");
        // DB Sanity version
        printInfoRow("dbsanity_version",  VersionInfo.getInfo("dbsanity").getVersion(), out);
        // app version
        Object appVersion = context.getAppVersion();
        if (appVersion != null && !"*".equals(appVersion.toString().trim()))
            printInfoRow("application_version", appVersion.toString(), out);
        // URL
        printInfoRow("url",  connectData.url, out);
        // User
        printInfoRow("user", connectData.user, out);
        // Table restriction
        if (!CollectionUtil.isEmpty(context.getTablesToCheck()))
        	printInfoRow("table_restriction", ArrayFormat.format(context.getTablesToCheck().toArray()), out);
        // Tag restriction
        if (!CollectionUtil.isEmpty(context.getTags()))
        	printInfoRow("tag_restriction", ArrayFormat.format(context.getTags().toArray()), out);
        // Defect count restriction
        if (context.getDefectCountLimit() > 0)
        	printInfoRow("defect_count_limit", String.valueOf(context.getDefectCountLimit()), out);
        if (context.getDefectRowLimit() > 0)
        	printInfoRow("defect_count_limit", String.valueOf(context.getDefectRowLimit()), out);
        // Ignorable tables
        printInfoRow("ignorable_tables", renderBreakableRegex(context.getIgnorableTables()), out);
        // condensed mode
        if (context.isCondensed())
        	printInfoRow("mode", "Condensed", out);
        // Command line params
        printInfoRow("invocation_params", (String) context.get("commandLine"), out);
        out.endTable();
	}

	private String renderBreakableRegex(String pattern) {
		if (StringUtil.isEmpty(pattern))
			return pattern;
		return pattern.replace("|", "|<wbr/>");
	}

	protected void renderStats(SanityCheckSuite suite, HtmlPrintWriter out) {
		out.printModuleSeparator("Stats");
        out.startTable("cellpadding", "3");
        // Started
        printInfoRow("started",  context.formatDate(suite.getStartDate()), out);
        // Duration
        printInfoRow("duration", TimeUtil.formatDuration(suite.getExecutionTime(), true, false), out);
        out.endTable();
	}

	protected void renderCheckedTags(SanityCheckSuite suite, HtmlPrintWriter out) {
		if (!CollectionUtil.isEmpty(context.getTags())) { // render tags only if they were restricted
			out.printModuleSeparator(localize("checked_tags"));
		    CheckTagAggregator visitor = new CheckTagAggregator(
		    		context.isUsingNamespaces(), context.isAuthorInCloud(), context.isTableInCloud()) {
				@Override
				protected long weight(SanityCheck check) {
					return 1;
				}
			};
			suite.accept(visitor);
		    Map<String, Long> weights = visitor.getWeights();
			for (String tag : weights.keySet()) {
				out.print(tag);
				out.print(" ");
			}
		}
	}

	protected void printInfoRow(String header, String value, HtmlPrintWriter out) {
        out.render(row(
            headerCell(localize(header)).leftAligned(),
            cell(value).leftAligned()
        ).topAligned());
        out.println();
	}

}
