/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Date;
import java.util.HashSet;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.Context;
import org.databene.commons.ConversionException;
import org.databene.commons.context.DefaultContext;
import org.databene.commons.converter.ThreadSafeConverter;
import org.databene.commons.time.ElapsedTimeFormatter;
import org.databene.commons.version.VersionInfo;
import org.databene.commons.version.VersionNumber;
import org.databene.dbsanity.ExecutionMode;
import org.databene.dbsanity.html.HtmlEntity;
import org.databene.dbsanity.html.HtmlFileImage;
import org.databene.dbsanity.html.HtmlFileReference;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.html.HtmlUrlReference;
import org.databene.dbsanity.html.Renderable;
import org.databene.dbsanity.html.RenderableMessageFormat;
import org.databene.dbsanity.html.RenderableText;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * {@link Context} implementation that provides data and operations for report generation.<br/><br/>
 * Created: 29.11.2010 14:47:03
 * @since 0.4
 * @author Volker Bergmann
 */
public class ReportContext extends DefaultContext {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(ReportContext.class);

	public static final String RED    = "EE0000";
	public static final String GREEN  = "00CC00";
	public static final String ORANGE = "EE6600";
	public static final String BLACK  = "AA0000";
	
	public static final String DEFECT_COLOR = "AA0000";
	public static final String HELP_KEY = "help";
	public static final String MORE = "more";
	
	private String rootSuiteName;
	private File reportRoot;
	VersionNumber appVersion;
	private DecimalFormat dnf;
	private DecimalFormat inf;
	private DateFormat dateFormat;
	private DurationFormatter durationFormatter;
	private Set<String> tablesToCheck;
	private String ignorableTables;
	private Set<String> tags;
	private int defectCountLimit;
	private int defectRowLimit;
	private boolean usingNamespaces = false;
	private boolean authorInCloud = true;
	private boolean tableInCloud = false;
	private boolean condensed;
	private ResourceBundle resourceBundle;
	
	public ReportContext(String rootSuiteName, File reportRoot, String[] tables, Locale locale) {
		this.rootSuiteName = rootSuiteName;
		this.reportRoot = reportRoot;
		this.dnf = new DecimalFormat("#,##0.0", DecimalFormatSymbols.getInstance(locale));
		this.inf = new DecimalFormat("#,##0", DecimalFormatSymbols.getInstance(locale));
		this.dateFormat = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, locale);
		this.durationFormatter = new DurationFormatter(locale);
		this.tablesToCheck = new HashSet<String>();
		this.tags = new HashSet<String>();
		this.defectCountLimit = 0;
		this.defectRowLimit = 0;
		set("executionMode", ExecutionMode.DEFAULT);
		this.resourceBundle = ResourceBundle.getBundle("dbsanity", locale);
		this.appVersion = null;
	}
	
		
	// properties ------------------------------------------------------------------------------------------------------
	
	public File getReportRoot() {
		return reportRoot;
	}
	
	public VersionNumber getAppVersion() {
		return appVersion;
	}
	
	public void setAppVersion(VersionNumber appVersion) {
		this.appVersion = appVersion;
	}

	public boolean isUsingNamespaces() {
		return usingNamespaces;
	}

	public void setUsingNamespaces(boolean usingNamespaces) {
		this.usingNamespaces = usingNamespaces;
	}
	
	public boolean isAuthorInCloud() {
		return authorInCloud;
	}

	public void setAuthorInCloud(boolean authorInCloud) {
		this.authorInCloud = authorInCloud;
	}
	
	public boolean isTableInCloud() {
		return tableInCloud;
	}

	public void setTableInCloud(boolean tableInCloud) {
		this.tableInCloud = tableInCloud;
	}
	
	public Set<String> getTablesToCheck() {
		return tablesToCheck;
	}
	
	public void setTablesToCheck(String[] tablesToCheck) {
		this.tablesToCheck = CollectionUtil.toSet(tablesToCheck);
	}

	public String getIgnorableTables() {
		return ignorableTables;
	}
	
	public void setIgnorableTables(String ignorableTables) {
		this.ignorableTables = ignorableTables;
	}

	public void setTags(Set<String> tags) {
		this.tags = tags;
	}

	public Set<String> getTags() {
		return tags;
	}

	public int getDefectCountLimit() {
		return defectCountLimit;
	}
	
	public void setDefectCountLimit(int defectCountLimit) {
		this.defectCountLimit = defectCountLimit;
	}
	
	public int getDefectRowLimit() {
		return defectRowLimit;
	}
	
	public void setDefectRowLimit(int defectRowLimit) {
		this.defectRowLimit = defectRowLimit;
	}
	
	public boolean isCondensed() {
		return condensed;
	}
	
	public void setCondensed(boolean condensed) {
		this.condensed = condensed;
	}
	
	
	// functional interface --------------------------------------------------------------------------------------------
	
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	
	public String localize(String key) {
		try {
			return resourceBundle.getString(key);
		} catch (Exception e) {
			LOGGER.error("Error retrieving key " + key, e);
			return key; 
		}
	}
	
	public String formatLong(long number) {
		return inf.format(number);
	}
	
	public String formatDecimal(double number) {
		return dnf.format(number);
	}
	
	public String formatLong(long number, String zeroText) {
		return (number > 0 ? formatLong(number) : zeroText);
	}

	public Renderable longRenderable(long number, Renderable zeroRenderable) {
		return (number > 0 ? new RenderableText(formatLong(number)) : zeroRenderable);
	}

	public Renderable formatDuration(long executionTime) {
		return durationFormatter.convert(executionTime);
	}

	public String formatDate(Date date) {
		return (date != null ? dateFormat.format(date) : "");
	}

	public void printNavBar(File forFileOrFolder, HtmlPrintWriter out, HtmlFileReference... refs) {
		out.startPanel("class", "nav_bar");
		out.startTable("width", "100%");
		out.startTableRow();
		out.startTableCell();
		formatBreadcrumbs(forFileOrFolder, forFileOrFolder, out);
		for (int i = 0; i < refs.length; i += 2) {
			out.print(" | ");
			out.render(refs[i]);
		}
		out.endTableCell();
		out.startTableCell("align", "right");
		String helpLabel = localize(ReportContext.HELP_KEY);
		out.render(new HtmlUrlReference(helpLabel, "http://databene.org/dbsanity").withTarget("_blank"));
		out.endTableCell();
		out.endTableRow();
		out.endTable();
		out.endPanel();
	}
	
	private void formatBreadcrumbs(File fromFile, File toFile, HtmlPrintWriter out) {
		if (!toFile.equals(reportRoot)) {
			formatBreadcrumbs(fromFile, toFile.getParentFile(), out);
			out.print(" / ");
		}
		String label = (toFile.equals(reportRoot) ? rootSuiteName : toFile.getName());
		if (fromFile.equals(toFile)) {
			out.startBold();
			out.print(label);
			out.endBold();
		} else if (toFile.isDirectory())
			out.render(new HtmlFileReference(label, new File(toFile, "index.html")));
		else
			out.render(new HtmlFileReference(label, toFile));
	}

	public HtmlFileImage verdictImage(SanityCheckVerdict verdict) {
		String verdictName = verdict.name().toLowerCase();
		String fileName = verdictName + "-16.gif";
		File imgFile = new File(reportRoot, fileName);
		return new HtmlFileImage(imgFile).withTitle(verdictName);
	}

	public void renderFooter(HtmlPrintWriter out) {
		out.startParagraph("class", "orange");
		out.startBold();
		String headerPattern = localize("footer");
		HtmlUrlReference vbRef = new HtmlUrlReference("Volker Bergmann", "http://bergmann-it.de").withTarget("_blank").withStyle("color:#AA4400;");
		HtmlUrlReference databeneRef = new HtmlUrlReference("Databene", "http://databene.org").withTarget("_blank").withStyle("color:#AA4400;");
		String appName = "DB Sanity " + VersionInfo.getInfo("dbsanity").getVersion();
		HtmlUrlReference dbSanityRef = new HtmlUrlReference(appName, "http://databene.org/dbsanity").withTarget("_blank").withStyle("color:#AA4400;");
		RenderableMessageFormat.format(headerPattern, out, vbRef, databeneRef, dbSanityRef);
		out.endBold();
		out.endParagraph();
	}
	
	static class DurationFormatter extends ThreadSafeConverter<Long, Renderable> {
		
		private ElapsedTimeFormatter stringFormatter;
		
		public DurationFormatter(Locale locale) {
			super(Long.class, Renderable.class);
			this.stringFormatter = new ElapsedTimeFormatter(locale, " ");
		}

		public Renderable convert(Long millis) throws ConversionException {
			if (millis != null)
				return new RenderableText(stringFormatter.convert(millis));
			else
				return new HtmlEntity("nbsp");
		}
		
	}

}
