/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.databene.commons.FileUtil;
import org.databene.commons.IOUtil;
import org.databene.commons.StringCharacterIterator;
import org.databene.commons.SystemInfo;
import org.databene.dbsanity.html.HtmlDiv;
import org.databene.dbsanity.html.HtmlFileReference;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.html.HtmlTableCell;
import org.databene.dbsanity.html.Renderable;
import org.databene.dbsanity.html.RenderableText;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.dbsanity.util.CheckAggregator;

/**
 * Provides utility methods for report generation.<br/><br/>
 * Created: 14.10.2010 16:14:08
 * @since 1.0
 * @author Volker Bergmann
 */
public class ReportUtil { // TODO v0.x move methods to ReportContext and design common concept for dbsanity and mad4db

	public static String normalizeUrl(String rawUrl) {
		StringBuilder url = new StringBuilder(rawUrl.length());
		StringCharacterIterator iterator = new StringCharacterIterator(rawUrl);
		while (iterator.hasNext()) {
			char c = iterator.next();
			if (Character.isLetterOrDigit(c) || c=='.' || c == '-'|| c == '_'|| c == '+')
				url.append(c);
			else if (c == '*')
				url.append('x');
			else if (c == ' ')
				url.append('_');
			// all other characters are ignored
		}
		return url.toString();
	}
	
	public static Renderable summaryBar(SanityCheckSuite suite, String width) {
		return new HtmlSummaryRenderable(suite, width);
	}

	public static double getSkipPercentage(SanityCheckSuite suite) {
		if (getTestCount(suite) > 0)
			return suite.countSkippedChecks() * 100. / getTestCount(suite);
		else return 0;
	}

	public static double getSuccessPercentage(SanityCheckSuite suite) {
		if (getTestCount(suite) > 0)
			return suite.countPassedChecks() * 100. / getTestCount(suite);
		else return 0;
	}

	public static double getFailurePercentage(SanityCheckSuite suite) {
		if (getTestCount(suite) > 0)
			return suite.countFailedChecks() * 100. / getTestCount(suite);
		else return 0;
	}

	public static double getErrorPercentage(SanityCheckSuite suite) {
		if (getTestCount(suite) > 0)
			return suite.countErredChecks() * 100 / getTestCount(suite);
		else
			return 0;
	}

	public static int getTestCount(SanityCheckSuite suite) {
		return suite.countPassedChecks() 
				+ suite.countFailedChecks()
				+ suite.countErredChecks()
				+ suite.countSkippedChecks();
	}

	public static HtmlFileReference docRef(SanityCheck check, ReportContext context) {
		return docRef(check, context.localize("label_source_file"));
	}

	public static HtmlFileReference docRef(SanityCheck check, String label) {
		return new HtmlFileReference(label, check.getDocPage(), check.getName());
	}

	public static List<SanityCheck> aggregateChecks(SanityCheckSuite suite, boolean onlyApplicables) {
		CheckAggregator visitor = new CheckAggregator(onlyApplicables);
		suite.accept(visitor);
		return visitor.getChecks();
	}

	public static void copyResourceFiles(File targetFolder, String style) throws IOException {
		FileUtil.ensureDirectoryExists(targetFolder);
		copyResourceFile("dbsanity-logo-64.gif", targetFolder);
		copyResourceFile("not_applicable-16.gif", targetFolder);
		copyResourceFile("pending-16.gif", targetFolder);
		copyResourceFile("passed-16.gif", targetFolder);
		copyResourceFile("failed-16.gif", targetFolder);
		copyResourceFile("error-16.gif", targetFolder);
		copyResourceFile("skipped-16.gif", targetFolder);
		copyResourceFile("red.gif", targetFolder);
		copyResourceFile("green.gif", targetFolder);
		copyResourceFile("blue.gif", targetFolder);
		copyResourceFile("black.gif", targetFolder);
		copyResourceFile("gray.gif", targetFolder);
		copyResourceFile("favicon.ico", targetFolder);
		copyResourceFile(style + ".css", targetFolder);
	}

	public static void copyResourceFile(String filename, File folder) throws IOException {
		IOUtil.copyFile(filename, folder.getCanonicalPath() + SystemInfo.getFileSeparator() + filename);
	}
	
	public static HtmlFileReference docOrDefectFileRef(SanityCheck check) {
		return docOrDefectFileRef(check, new RenderableText(check.getName()));
	}

	public static HtmlFileReference docOrDefectFileRef(SanityCheck check, Renderable label) {
		SanityCheckVerdict verdict = check.getVerdict();
		if (verdict == SanityCheckVerdict.ERROR || verdict == SanityCheckVerdict.FAILED)
			return new HtmlFileReference(label, check.getDefectsOrErrorPage());
		else
			return new HtmlFileReference(label, check.getDocPage(), ReportUtil.normalizeUrl(check.getName()));
	}

	public static class HtmlSummaryRenderable implements Renderable {

		SanityCheckSuite suite;
		String width;
		
		public HtmlSummaryRenderable(SanityCheckSuite suite, String width) {
			this.suite = suite;
			this.width = width;
		}

		public void render(HtmlPrintWriter out) {
			out.startTable("width", String.valueOf(width), "cellpadding", "0", "cellspacing", "0");
			out.startTableRow("align", "left");
			renderBarPart(suite.countErredChecks(), getErrorPercentage(suite), "000000", "Checks with errors", out);
			renderBarPart(suite.countFailedChecks(), getFailurePercentage(suite), ReportContext.RED, "Failed checks", out);
			renderBarPart(suite.countPassedChecks(), getSuccessPercentage(suite), ReportContext.GREEN, "Passed checks", out);
			renderBarPart(suite.countSkippedChecks(), getSkipPercentage(suite), "BBBBBB", "Skipped checks", out);
			out.endTableRow();
			out.endTable();
		}

		private void renderBarPart(int count, double percentage, String color, String title, HtmlPrintWriter out) {
			if (count > 0) {
				String style = "background: #" + color + "; color: #ffffff;";
				String partWidth = ((int) percentage) + "%";
				out.render(new HtmlTableCell().withWidth(partWidth).withContent(
						new HtmlDiv().withTitle(title).withStyle(style).withContent(String.valueOf(count))
					)
				);
			}
		}

	}

}
