/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.databene.commons.ConfigurationError;
import org.databene.commons.IOUtil;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.jdbacl.SQLUtil;

/**
 * Creates a summary report in Excel(TM) format.<br/><br/>
 * Created: 23.01.2012 09:43:00
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class XLSSummaryModule extends AbstractSummaryReportModule<XLSWrapper> {
	
	private static final String XLS_REPORT_FILE_NAME = "dbsanity-report-summary.xls";
	
	private int rownum;
	
	public XLSSummaryModule() {
		super("xls", XLS_REPORT_FILE_NAME);
		this.rownum = 0;
	}
	
	@Override
	protected XLSWrapper startSummaryFile(SanityCheckSuite rootSuite) throws IOException {
		File file = getFile(rootSuite);
		HSSFWorkbook workbook = new HSSFWorkbook();
		HSSFSheet sheet = workbook.createSheet();
		String check = context.localize("header_check");
		String table = context.localize("header_table");
		String defectType = context.localize("header_defect_type");
		String verdict = context.localize("header_verdict");
		String defectCount = context.localize("header_defect_count");
		String duration = context.localize("header_duration");
		String details = context.localize("header_details");
		String[] headers =  { check, table, defectType, verdict, defectCount, duration, details };
	    HSSFRow headerRow = sheet.createRow(rownum++);
	    for (int i = 0; i < headers.length; i++)
			headerRow.createCell(i).setCellValue(new HSSFRichTextString(headers[i]));
		return new XLSWrapper(file, sheet);
	}

	@Override
	protected void exportCheck(SanityCheck check, XLSWrapper xls) {
		String details = check.getDetails();
		String normalizedQuery = (details != null ? SQLUtil.normalize(details, true) : null);
	    HSSFRow headerRow = xls.sheet.createRow(rownum++);
	    int cellnum = 0;
		headerRow.createCell(cellnum++).setCellValue(new HSSFRichTextString(check.getName()));
		headerRow.createCell(cellnum++).setCellValue(new HSSFRichTextString(check.getTable()));
		headerRow.createCell(cellnum++).setCellValue(new HSSFRichTextString(check.getDefectType()));
		headerRow.createCell(cellnum++).setCellValue(new HSSFRichTextString(check.getVerdict().name()));
		headerRow.createCell(cellnum++).setCellValue(check.getDefectCount());
		headerRow.createCell(cellnum++).setCellValue(check.getExecutionTime());
 		headerRow.createCell(cellnum++).setCellValue(new HSSFRichTextString(normalizedQuery));
	}
	
	@Override
	protected void endSummaryFile(XLSWrapper xls) {
		HSSFSheet sheet = xls.sheet;
		for (int colnum = 0; colnum <= sheet.getLastRowNum(); colnum++)
			sheet.autoSizeColumn(colnum);
		HSSFWorkbook workbook = sheet.getWorkbook();
		FileOutputStream out = null;
        try {
            out = new FileOutputStream(xls.file);
			workbook.write(out);
		} catch (FileNotFoundException e) {
			throw new ConfigurationError(e);
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			IOUtil.close(out);
		}
	}

}

class XLSWrapper {
	public final File file;
	public final HSSFSheet sheet;

	public XLSWrapper(File file, HSSFSheet sheet) {
		this.file = file;
		this.sheet = sheet;
	}
}
