/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.databene.commons.collection.OrderedNameMap;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.html.HtmlSpan;
import org.databene.script.WeightUtil;
import org.databene.script.WeightedSample;

/**
 * Provides tag cloud related functionality.<br/><br/>
 * Created: 11.11.2011 17:13:08
 * @since 0.9.2
 * @author Volker Bergmann
 */
public class TagCloudUtil {

	public static void printTagCloud(Map<String, Long> weights, 
			String color, int minFontSize, int maxFontSize, HtmlPrintWriter out) {
		TagCloud cloud = new TagCloud();
		for (Map.Entry<String, Long> entry : weights.entrySet())
			cloud.addTagWeight(entry.getKey(), entry.getValue());
		List<WeightedSample<String>> fontSizes = cloud.calculateFontSizes(minFontSize, maxFontSize, false);
		out.startTag("font", "color", color);
		for (int i = 0; i < fontSizes.size(); i++) {
			if (i > 0 )
				out.print(" ");
			WeightedSample<String> tag = fontSizes.get(i);
			String fontStyle = "font-size:" + (int) tag.getWeight() + "%;";
			out.render(new HtmlSpan().withStyle(fontStyle).withContent(tag.getValue()));
		}
		out.endTag("font");
	}

	private static class TagCloud {

		private Map<String, WeightedSample<String>> tagWeights;
		private double maxWeight;
		
		public TagCloud() {
			this.tagWeights = OrderedNameMap.<WeightedSample<String>>createCaseInsensitiveMap();
			this.maxWeight = 0;
		}
		
		public void addTagWeight(String tag, double diffWeight) {
			WeightedSample<String> tagWeight = tagWeights.get(tag);
			if (tagWeight == null) {
				tagWeight = new WeightedSample<String>(tag, diffWeight);
				tagWeights.put(tag, tagWeight);
			} else {
				tagWeight.addWeight(diffWeight);
			}
			if (tagWeight.getWeight() > maxWeight)
				maxWeight = tagWeight.getWeight();
		}
		
		public List<WeightedSample<String>> calculateFontSizes(int minSize, int maxSize, boolean ordered) {
			// calculate min weight
			double minWeight = maxWeight;
			for (WeightedSample<String> tag : tagWeights.values()) {
				if (tag.getWeight() < minWeight)
					minWeight = tag.getWeight();
			}
			// derive font sizes
			double range = maxWeight - minWeight;
			List<WeightedSample<String>> fontSizes = new ArrayList<WeightedSample<String>>(tagWeights.size());
			for (WeightedSample<String> tag : tagWeights.values()) {
				if (range == 0)
					fontSizes.add(new WeightedSample<String>(tag.getValue(), minSize));
				else {
					int fontSize = (int) (minSize + (maxSize - minSize) * Math.log(1 + (Math.E - 1) * (tag.getWeight() - minWeight) / range));
					fontSizes.add(new WeightedSample<String>(tag.getValue(), fontSize));
				}
			}
			if (ordered)
				WeightUtil.sortByWeight(fontSizes, false);
			return fontSizes;
		}
	}
}
